from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_event_streams import cs_alert_context


@panther_managed
class CrowdstrikeEppDetectionSummary(Rule):
    id = "Crowdstrike.EppDetectionSummary-prototype"
    display_name = "Crowdstrike Detection Summary"
    log_types = [LogType.CROWDSTRIKE_EVENT_STREAMS]
    default_severity = Severity.MEDIUM
    default_description = "Forwards any alerts generated by CrowdStrike to your Panther destinations."
    default_runbook = "Followup with the alert according to CrowdStrike's recommendations."
    tags = ["Crowdstrike", "Passthrough"]

    def rule(self, event: PantherEvent):
        return event.deep_get("metadata", "eventType") == "EppDetectionSummaryEvent"

    def title(self, event: PantherEvent):
        alert_title = event.deep_get("event", "Name", default="New CrowdStrike Detection")
        alert_desc = event.deep_get("event", "Description")
        return f"{alert_title}: {alert_desc}" if alert_desc else alert_title

    def dedup(self, event: PantherEvent):
        if alert_id := event.deep_get("event", "CompositeId"):
            return alert_id
        # Else, fall back on title string
        return self.title(event)

    def severity(self, event: PantherEvent):
        # First, try returning the severity based on the SeverityName
        sevname = event.deep_get("event", "SeverityName").upper()
        allowed_values = ("INFO", "LOW", "MEDIUM", "HIGH", "CRITICAL")
        if sevname == "INFORMATIONAL":
            sevname = "INFO"
        if sevname in allowed_values:
            return sevname
        # Else, fallback on the numerical value, falling back on MEDIUM if we still don't have a value
        sevval = event.deep_get("event", "Severity") // 20
        return {0: "INFO", 1: "LOW", 2: "MEDIUM", 3: "HIGH", 4: "CRITICAL", 5: "CRITICAL"}.get(sevval, "DEFAULT")

    def reference(self, event: PantherEvent):
        return event.deep_get("event", "FalconHostLink", default="")

    def alert_context(self, event: PantherEvent):
        context = cs_alert_context(event)
        context.update(
            {
                "FalconLink": event.deep_get("event", "FalconHostLink", default="<NO LINK PROVIDED>"),
                "CompositeId": event.deep_get("event", "CompositeId", default="<NO ID PROVIDED>"),
                "FileName": event.deep_get("event", "FileName", default="<NO FILENAME PROVIDED>"),
                "FilePath": event.deep_get("event", "FilePath", default="<NO FILEPATH PROVIDED>"),
                "UserName": event.deep_get("event", "UserName", default="<NO USERNAME PROVIDED>"),
            },
        )
        return context

    tests = [
        RuleTest(
            name="INFO Alert",
            expected_result=True,
            log={
                "p_event_time": "2024-12-03 19:54:30.000000000",
                "p_log_type": "Crowdstrike.EventStreams",
                "p_parse_time": "2024-12-03 19:55:53.866056859",
                "event": {
                    "CommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --type=utility --utility-sub-type=chrome.mojom.UtilWin --lang=en-US --service-sandbox-type=none --string-annotations=is-enterprise-managed=no --field-trial-handle=3832,i,16271085821438557631,11283940248133486999,262144 --variations-seed-version --mojo-platform-channel-handle=3900 /prefetch:8',
                    "CompositeId": "DETECTION ID",
                    "Description": "A process has written a known EICAR test file. Review the files written by the triggered process.",
                    "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity-v2/detections/DETECTION-ID?_cid=g04000rdir6xllkvfbjftn5dbo67ord4",
                    "FileName": "msedge.exe",
                    "FilePath": "\\Device\\HarddiskVolume1\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe",
                    "GrandparentCommandLine": "C:\\Windows\\Explorer.EXE",
                    "GrandparentImageFilename": "explorer.exe",
                    "LocalIP": "10.0.6.37",
                    "MACAddress": "06-a7-e1-13-09-0d",
                    "MD5String": "fcde6b30b89cabf7d0460bc5a580cb12",
                    "Name": "Known Malware",
                    "Objective": "Follow Through",
                    "ParentCommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --profile-directory=Default',
                    "ParentImageFileName": "msedge.exe",
                    "ParentProcessId": "90559720124",
                    "PatternDispositionDescription": "Detection, standard detection.",
                    "PatternDispositionFlags": {
                        "BootupSafeguardEnabled": False,
                        "CriticalProcessDisabled": False,
                        "Detect": False,
                        "FsOperationBlocked": False,
                        "InddetMask": False,
                        "Indicator": False,
                        "KillParent": False,
                        "KillProcess": False,
                        "KillSubProcess": False,
                        "OperationBlocked": False,
                        "PolicyDisabled": False,
                        "ProcessBlocked": False,
                        "QuarantineFile": False,
                        "QuarantineMachine": False,
                        "RegistryOperationBlocked": False,
                        "Rooting": False,
                        "SensorOnly": False,
                    },
                    "PatternDispositionValue": 0,
                    "ProcessEndTime": "1970-01-01 00:00:00.000000000",
                    "ProcessId": "90597916092",
                    "ProcessStartTime": "2024-12-03 19:53:19.000000000",
                    "SHA1String": "0000000000000000000000000000000000000000",
                    "SHA256String": "05d8bce07453a2f3e77503797e39365fd78766099ca60968b9f411ce9886d4ae",
                    "Severity": 10,
                    "SeverityName": "Informational",
                    "Tactic": "Execution",
                    "Technique": "User Execution",
                    "UserName": "Administrator",
                },
                "metadata": {
                    "customerIDString": "CUST_ID",
                    "eventCreationTime": "2024-12-03 19:54:30.000000000",
                    "eventType": "EppDetectionSummaryEvent",
                    "offset": 1201171,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="LOW Alert",
            expected_result=True,
            log={
                "p_event_time": "2024-12-03 19:54:30.000000000",
                "p_log_type": "Crowdstrike.EventStreams",
                "p_parse_time": "2024-12-03 19:55:53.866056859",
                "event": {
                    "CommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --type=utility --utility-sub-type=chrome.mojom.UtilWin --lang=en-US --service-sandbox-type=none --string-annotations=is-enterprise-managed=no --field-trial-handle=3832,i,16271085821438557631,11283940248133486999,262144 --variations-seed-version --mojo-platform-channel-handle=3900 /prefetch:8',
                    "CompositeId": "DETECTION ID",
                    "Description": "A process has written a known EICAR test file. Review the files written by the triggered process.",
                    "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity-v2/detections/DETECTION-ID?_cid=g04000rdir6xllkvfbjftn5dbo67ord4",
                    "FileName": "msedge.exe",
                    "FilePath": "\\Device\\HarddiskVolume1\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe",
                    "GrandparentCommandLine": "C:\\Windows\\Explorer.EXE",
                    "GrandparentImageFilename": "explorer.exe",
                    "LocalIP": "10.0.6.37",
                    "MACAddress": "06-a7-e1-13-09-0d",
                    "MD5String": "fcde6b30b89cabf7d0460bc5a580cb12",
                    "Name": "Known Malware",
                    "Objective": "Follow Through",
                    "ParentCommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --profile-directory=Default',
                    "ParentImageFileName": "msedge.exe",
                    "ParentProcessId": "90559720124",
                    "PatternDispositionDescription": "Detection, standard detection.",
                    "PatternDispositionFlags": {
                        "BootupSafeguardEnabled": False,
                        "CriticalProcessDisabled": False,
                        "Detect": False,
                        "FsOperationBlocked": False,
                        "InddetMask": False,
                        "Indicator": False,
                        "KillParent": False,
                        "KillProcess": False,
                        "KillSubProcess": False,
                        "OperationBlocked": False,
                        "PolicyDisabled": False,
                        "ProcessBlocked": False,
                        "QuarantineFile": False,
                        "QuarantineMachine": False,
                        "RegistryOperationBlocked": False,
                        "Rooting": False,
                        "SensorOnly": False,
                    },
                    "PatternDispositionValue": 0,
                    "ProcessEndTime": "1970-01-01 00:00:00.000000000",
                    "ProcessId": "90597916092",
                    "ProcessStartTime": "2024-12-03 19:53:19.000000000",
                    "SHA1String": "0000000000000000000000000000000000000000",
                    "SHA256String": "05d8bce07453a2f3e77503797e39365fd78766099ca60968b9f411ce9886d4ae",
                    "Severity": 20,
                    "SeverityName": "Low",
                    "Tactic": "Execution",
                    "Technique": "User Execution",
                    "UserName": "Administrator",
                },
                "metadata": {
                    "customerIDString": "CUST_ID",
                    "eventCreationTime": "2024-12-03 19:54:30.000000000",
                    "eventType": "EppDetectionSummaryEvent",
                    "offset": 1201171,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="MEDIUM Alert",
            expected_result=True,
            log={
                "p_event_time": "2024-12-03 19:54:30.000000000",
                "p_log_type": "Crowdstrike.EventStreams",
                "p_parse_time": "2024-12-03 19:55:53.866056859",
                "event": {
                    "CommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --type=utility --utility-sub-type=chrome.mojom.UtilWin --lang=en-US --service-sandbox-type=none --string-annotations=is-enterprise-managed=no --field-trial-handle=3832,i,16271085821438557631,11283940248133486999,262144 --variations-seed-version --mojo-platform-channel-handle=3900 /prefetch:8',
                    "CompositeId": "DETECTION ID",
                    "Description": "A process has written a known EICAR test file. Review the files written by the triggered process.",
                    "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity-v2/detections/DETECTION-ID?_cid=g04000rdir6xllkvfbjftn5dbo67ord4",
                    "FileName": "msedge.exe",
                    "FilePath": "\\Device\\HarddiskVolume1\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe",
                    "GrandparentCommandLine": "C:\\Windows\\Explorer.EXE",
                    "GrandparentImageFilename": "explorer.exe",
                    "LocalIP": "10.0.6.37",
                    "MACAddress": "06-a7-e1-13-09-0d",
                    "MD5String": "fcde6b30b89cabf7d0460bc5a580cb12",
                    "Name": "Known Malware",
                    "Objective": "Follow Through",
                    "ParentCommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --profile-directory=Default',
                    "ParentImageFileName": "msedge.exe",
                    "ParentProcessId": "90559720124",
                    "PatternDispositionDescription": "Detection, standard detection.",
                    "PatternDispositionFlags": {
                        "BootupSafeguardEnabled": False,
                        "CriticalProcessDisabled": False,
                        "Detect": False,
                        "FsOperationBlocked": False,
                        "InddetMask": False,
                        "Indicator": False,
                        "KillParent": False,
                        "KillProcess": False,
                        "KillSubProcess": False,
                        "OperationBlocked": False,
                        "PolicyDisabled": False,
                        "ProcessBlocked": False,
                        "QuarantineFile": False,
                        "QuarantineMachine": False,
                        "RegistryOperationBlocked": False,
                        "Rooting": False,
                        "SensorOnly": False,
                    },
                    "PatternDispositionValue": 0,
                    "ProcessEndTime": "1970-01-01 00:00:00.000000000",
                    "ProcessId": "90597916092",
                    "ProcessStartTime": "2024-12-03 19:53:19.000000000",
                    "SHA1String": "0000000000000000000000000000000000000000",
                    "SHA256String": "05d8bce07453a2f3e77503797e39365fd78766099ca60968b9f411ce9886d4ae",
                    "Severity": 50,
                    "SeverityName": "Medium",
                    "Tactic": "Execution",
                    "Technique": "User Execution",
                    "UserName": "Administrator",
                },
                "metadata": {
                    "customerIDString": "CUST_ID",
                    "eventCreationTime": "2024-12-03 19:54:30.000000000",
                    "eventType": "EppDetectionSummaryEvent",
                    "offset": 1201171,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="HIGH Alert",
            expected_result=True,
            log={
                "p_event_time": "2024-12-03 19:54:30.000000000",
                "p_log_type": "Crowdstrike.EventStreams",
                "p_parse_time": "2024-12-03 19:55:53.866056859",
                "event": {
                    "CommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --type=utility --utility-sub-type=chrome.mojom.UtilWin --lang=en-US --service-sandbox-type=none --string-annotations=is-enterprise-managed=no --field-trial-handle=3832,i,16271085821438557631,11283940248133486999,262144 --variations-seed-version --mojo-platform-channel-handle=3900 /prefetch:8',
                    "CompositeId": "DETECTION ID",
                    "Description": "A process has written a known EICAR test file. Review the files written by the triggered process.",
                    "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity-v2/detections/DETECTION-ID?_cid=g04000rdir6xllkvfbjftn5dbo67ord4",
                    "FileName": "msedge.exe",
                    "FilePath": "\\Device\\HarddiskVolume1\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe",
                    "GrandparentCommandLine": "C:\\Windows\\Explorer.EXE",
                    "GrandparentImageFilename": "explorer.exe",
                    "LocalIP": "10.0.6.37",
                    "MACAddress": "06-a7-e1-13-09-0d",
                    "MD5String": "fcde6b30b89cabf7d0460bc5a580cb12",
                    "Name": "Known Malware",
                    "Objective": "Follow Through",
                    "ParentCommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --profile-directory=Default',
                    "ParentImageFileName": "msedge.exe",
                    "ParentProcessId": "90559720124",
                    "PatternDispositionDescription": "Detection, standard detection.",
                    "PatternDispositionFlags": {
                        "BootupSafeguardEnabled": False,
                        "CriticalProcessDisabled": False,
                        "Detect": False,
                        "FsOperationBlocked": False,
                        "InddetMask": False,
                        "Indicator": False,
                        "KillParent": False,
                        "KillProcess": False,
                        "KillSubProcess": False,
                        "OperationBlocked": False,
                        "PolicyDisabled": False,
                        "ProcessBlocked": False,
                        "QuarantineFile": False,
                        "QuarantineMachine": False,
                        "RegistryOperationBlocked": False,
                        "Rooting": False,
                        "SensorOnly": False,
                    },
                    "PatternDispositionValue": 0,
                    "ProcessEndTime": "1970-01-01 00:00:00.000000000",
                    "ProcessId": "90597916092",
                    "ProcessStartTime": "2024-12-03 19:53:19.000000000",
                    "SHA1String": "0000000000000000000000000000000000000000",
                    "SHA256String": "05d8bce07453a2f3e77503797e39365fd78766099ca60968b9f411ce9886d4ae",
                    "Severity": 70,
                    "SeverityName": "High",
                    "Tactic": "Execution",
                    "Technique": "User Execution",
                    "UserName": "Administrator",
                },
                "metadata": {
                    "customerIDString": "CUST_ID",
                    "eventCreationTime": "2024-12-03 19:54:30.000000000",
                    "eventType": "EppDetectionSummaryEvent",
                    "offset": 1201171,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="CRIT Alert",
            expected_result=True,
            log={
                "p_event_time": "2024-12-03 19:54:30.000000000",
                "p_log_type": "Crowdstrike.EventStreams",
                "p_parse_time": "2024-12-03 19:55:53.866056859",
                "event": {
                    "CommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --type=utility --utility-sub-type=chrome.mojom.UtilWin --lang=en-US --service-sandbox-type=none --string-annotations=is-enterprise-managed=no --field-trial-handle=3832,i,16271085821438557631,11283940248133486999,262144 --variations-seed-version --mojo-platform-channel-handle=3900 /prefetch:8',
                    "CompositeId": "DETECTION ID",
                    "Description": "A process has written a known EICAR test file. Review the files written by the triggered process.",
                    "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity-v2/detections/DETECTION-ID?_cid=g04000rdir6xllkvfbjftn5dbo67ord4",
                    "FileName": "msedge.exe",
                    "FilePath": "\\Device\\HarddiskVolume1\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe",
                    "GrandparentCommandLine": "C:\\Windows\\Explorer.EXE",
                    "GrandparentImageFilename": "explorer.exe",
                    "LocalIP": "10.0.6.37",
                    "MACAddress": "06-a7-e1-13-09-0d",
                    "MD5String": "fcde6b30b89cabf7d0460bc5a580cb12",
                    "Name": "Known Malware",
                    "Objective": "Follow Through",
                    "ParentCommandLine": '"C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe" --profile-directory=Default',
                    "ParentImageFileName": "msedge.exe",
                    "ParentProcessId": "90559720124",
                    "PatternDispositionDescription": "Detection, standard detection.",
                    "PatternDispositionFlags": {
                        "BootupSafeguardEnabled": False,
                        "CriticalProcessDisabled": False,
                        "Detect": False,
                        "FsOperationBlocked": False,
                        "InddetMask": False,
                        "Indicator": False,
                        "KillParent": False,
                        "KillProcess": False,
                        "KillSubProcess": False,
                        "OperationBlocked": False,
                        "PolicyDisabled": False,
                        "ProcessBlocked": False,
                        "QuarantineFile": False,
                        "QuarantineMachine": False,
                        "RegistryOperationBlocked": False,
                        "Rooting": False,
                        "SensorOnly": False,
                    },
                    "PatternDispositionValue": 0,
                    "ProcessEndTime": "1970-01-01 00:00:00.000000000",
                    "ProcessId": "90597916092",
                    "ProcessStartTime": "2024-12-03 19:53:19.000000000",
                    "SHA1String": "0000000000000000000000000000000000000000",
                    "SHA256String": "05d8bce07453a2f3e77503797e39365fd78766099ca60968b9f411ce9886d4ae",
                    "Severity": 90,
                    "SeverityName": "Critical",
                    "Tactic": "Execution",
                    "Technique": "User Execution",
                    "UserName": "Administrator",
                },
                "metadata": {
                    "customerIDString": "CUST_ID",
                    "eventCreationTime": "2024-12-03 19:54:30.000000000",
                    "eventType": "EppDetectionSummaryEvent",
                    "offset": 1201171,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "metadata": {
                    "customerIDString": "CUST_ID",
                    "eventCreationTime": "2024-12-03 19:54:30.000000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 1201171,
                    "version": "1.0",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.duo import deserialize_administrator_log_event_description, duo_alert_context


@panther_managed
class DuoAdminUserMFABypassEnabled(Rule):
    default_description = "An Administrator enabled a user to authenticate without MFA."
    display_name = "Duo Admin User MFA Bypass Enabled"
    default_reference = "https://duo.com/docs/policy#authentication-policy"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.User.MFA.Bypass.Enabled-prototype"

    def rule(self, event):
        if event.get("action") == "user_update":
            description = deserialize_administrator_log_event_description(event)
            if "status" in description:
                return description.get("status") == "Bypass"
        return False

    def title(self, event):
        return f"Duo: [{event.get('username', '<username_not_found>')}] updated account [{event.get('object', '<object_not_found>')}] to not require two-factor authentication."

    def alert_context(self, event):
        return duo_alert_context(event)

    tests = [
        RuleTest(
            name="Account Active",
            expected_result=False,
            log={
                "action": "user_update",
                "description": '{"status": "Active"}',
                "isotimestamp": "2021-10-05 22:45:33",
                "object": "bart.simpson@simpsons.com",
                "timestamp": "2021-10-05 22:45:33",
                "username": "Homer Simpson",
            },
        ),
        RuleTest(
            name="Account Disabled",
            expected_result=False,
            log={
                "action": "user_update",
                "description": '{"status": "Disabled"}',
                "isotimestamp": "2021-10-05 22:45:33",
                "object": "bart.simpson@simpsons.com",
                "timestamp": "2021-10-05 22:45:33",
                "username": "Homer Simpson",
            },
        ),
        RuleTest(
            name="Bypass Enabled",
            expected_result=True,
            log={
                "action": "user_update",
                "description": '{"status": "Bypass"}',
                "isotimestamp": "2021-10-05 22:45:33",
                "object": "bart.simpson@simpsons.com",
                "timestamp": "2021-10-05 22:45:33",
                "username": "Homer Simpson",
            },
        ),
        RuleTest(
            name="Phones Update",
            expected_result=False,
            log={
                "action": "user_update",
                "description": '{"phones": ""}',
                "isotimestamp": "2021-07-02 19:06:40",
                "object": "homer.simpson@simpsons.com",
                "timestamp": "2021-07-02 19:06:40",
                "username": "Homer Simpson",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIALogStreamingDisabled(Rule):
    id = "ZIA.Log.Streaming.Disabled-prototype"
    default_description = "This rule detects when ZIA log streaming was disabled."
    display_name = "ZIA Log Streaming Disabled"
    default_runbook = "Verify that this change was planned. If not, make sure to restore previous settings."
    default_reference = "https://help.zscaler.com/zia/about-nss-feeds"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        action = event.deep_get("event", "action", default="ACTION_NOT_FOUND")
        category = event.deep_get("event", "category", default="CATEGORY_NOT_FOUND")
        if action == "DELETE" and category == "NSS":
            return True
        return False

    def title(self, event):
        cloud_connection_url = event.deep_get(
            "event",
            "preaction",
            "cloudNssSiemConfiguration",
            "connectionURL",
            default="<CLOUD_CONNECTION_URL_NOT_FOUND>",
        )
        return f"[Zscaler.ZIA]: Log streaming for location [{cloud_connection_url}] was deleted by admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Log streaming disabled (NSS deleted)",
            expected_result=True,
            log={
                "event": {
                    "action": "DELETE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "NSS",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {},
                    "preaction": {
                        "cloudNss": True,
                        "cloudNssSiemConfiguration": {
                            "connectionHeaders": ["123:123"],
                            "connectionURL": "https://logs.company.net/http/a7adc684-f65c-42af-9519-0a0786656f20",
                            "lastSuccessFullTest": 0,
                            "maxBatchSize": 512,
                            "nssType": "NSS_FOR_WEB",
                            "oAuthAuthentication": False,
                            "siemType": "OTHER",
                            "testConnectivityCode": 0,
                        },
                        "customEscapedCharacter": ["ASCII_44", "ASCII_92", "ASCII_34"],
                        "duplicateLogs": 0,
                        "epsRateLimit": 0,
                        "feedOutputFormat": '\\{ "sourcetype" : "zscalernss-web", "event" : \\{"datetime":"%d{yy}-%02d{mth}-%02d{dd} %02d{hh}:%02d{mm}:%02d{ss}","reason":"%s{reason}","event_id":"%d{recordid}","protocol":"%s{proto}","action":"%s{action}","transactionsize":"%d{totalsize}","responsesize":"%d{respsize}","requestsize":"%d{reqsize}","urlcategory":"%s{urlcat}","serverip":"%s{sip}","requestmethod":"%s{reqmethod}","refererURL":"%s{ereferer}","useragent":"%s{eua}","product":"NSS","location":"%s{elocation}","ClientIP":"%s{cip}","status":"%s{respcode}","user":"%s{elogin}","url":"%s{eurl}","vendor":"Zscaler","hostname":"%s{ehost}","clientpublicIP":"%s{cintip}","threatcategory":"%s{malwarecat}","threatname":"%s{threatname}","filetype":"%s{filetype}","appname":"%s{appname}","app_status":"%s{app_status}","pagerisk":"%d{riskscore}","threatseverity":"%s{threatseverity}","department":"%s{edepartment}","urlsupercategory":"%s{urlsupercat}","appclass":"%s{appclass}","dlpengine":"%s{dlpeng}","urlclass":"%s{urlclass}","threatclass":"%s{malwareclass}","dlpdictionaries":"%s{dlpdict}","fileclass":"%s{fileclass}","bwthrottle":"%s{bwthrottle}","contenttype":"%s{contenttype}","unscannabletype":"%s{unscannabletype}","deviceowner":"%s{deviceowner}","devicehostname":"%s{devicehostname}","keyprotectiontype":"%s{keyprotectiontype}"\\}\\}\n',
                        "feedStatus": "ENABLED",
                        "id": 2898,
                        "jsonArrayToggle": True,
                        "name": "test-feed-2",
                        "nssFeedType": "JSON",
                        "nssFilter": {"securityFeedFilter": False},
                        "nssLogType": "WEBLOG",
                        "timeZone": "GMT",
                        "userObfuscation": "DISABLED",
                    },
                    "recordid": "371",
                    "resource": "test-feed-2",
                    "result": "SUCCESS",
                    "subcategory": "NSS_FEED",
                    "time": "2024-11-04 16:34:34.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="NSS created",
            expected_result=False,
            log={
                "event": {
                    "action": "CREATE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "NSS",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "cloudNss": True,
                        "cloudNssSiemConfiguration": {
                            "clientSecret": "******",
                            "connectionHeaders": ["123:123"],
                            "connectionURL": "https://logs.company.net/http/a7adc684-f65c-42af-9519-0a0786656f20",
                            "lastSuccessFullTest": 0,
                            "maxBatchSize": 512,
                            "nssType": "NSS_FOR_WEB",
                            "oAuthAuthentication": False,
                            "siemType": "OTHER",
                            "testConnectivityCode": 0,
                        },
                        "customEscapedCharacter": ["ASCII_44", "ASCII_92", "ASCII_34"],
                        "duplicateLogs": 0,
                        "epsRateLimit": 0,
                        "feedOutputFormat": '\\{ "sourcetype" : "zscalernss-web", "event" : \\{"datetime":"%d{yy}-%02d{mth}-%02d{dd} %02d{hh}:%02d{mm}:%02d{ss}","reason":"%s{reason}","event_id":"%d{recordid}","protocol":"%s{proto}","action":"%s{action}","transactionsize":"%d{totalsize}","responsesize":"%d{respsize}","requestsize":"%d{reqsize}","urlcategory":"%s{urlcat}","serverip":"%s{sip}","requestmethod":"%s{reqmethod}","refererURL":"%s{ereferer}","useragent":"%s{eua}","product":"NSS","location":"%s{elocation}","ClientIP":"%s{cip}","status":"%s{respcode}","user":"%s{elogin}","url":"%s{eurl}","vendor":"Zscaler","hostname":"%s{ehost}","clientpublicIP":"%s{cintip}","threatcategory":"%s{malwarecat}","threatname":"%s{threatname}","filetype":"%s{filetype}","appname":"%s{appname}","app_status":"%s{app_status}","pagerisk":"%d{riskscore}","threatseverity":"%s{threatseverity}","department":"%s{edepartment}","urlsupercategory":"%s{urlsupercat}","appclass":"%s{appclass}","dlpengine":"%s{dlpeng}","urlclass":"%s{urlclass}","threatclass":"%s{malwareclass}","dlpdictionaries":"%s{dlpdict}","fileclass":"%s{fileclass}","bwthrottle":"%s{bwthrottle}","contenttype":"%s{contenttype}","unscannabletype":"%s{unscannabletype}","deviceowner":"%s{deviceowner}","devicehostname":"%s{devicehostname}","keyprotectiontype":"%s{keyprotectiontype}"\\}\\}\n',
                        "feedStatus": "ENABLED",
                        "id": 2898,
                        "jsonArrayToggle": True,
                        "name": "test-feed-2",
                        "nssFeedType": "JSON",
                        "nssFilter": {"securityFeedFilter": False},
                        "nssLogType": "WEBLOG",
                        "timeZone": "GMT",
                        "userObfuscation": "DISABLED",
                    },
                    "preaction": {
                        "cloudNss": True,
                        "cloudNssSiemConfiguration": {
                            "connectionHeaders": ["123:123"],
                            "connectionURL": "https://logs.company.net/http/a7adc684-f65c-42af-9519-0a0786621f20",
                            "maxBatchSize": 524288,
                            "nssType": "NSS_FOR_WEB",
                            "oAuthAuthentication": False,
                            "siemType": "OTHER",
                        },
                        "customEscapedCharacter": ["ASCII_44", "ASCII_92", "ASCII_34"],
                        "duplicateLogs": 0,
                        "epsRateLimit": 0,
                        "feedOutputFormat": '\\{ "sourcetype" : "zscalernss-web", "event" : \\{"datetime":"%d{yy}-%02d{mth}-%02d{dd} %02d{hh}:%02d{mm}:%02d{ss}","reason":"%s{reason}","event_id":"%d{recordid}","protocol":"%s{proto}","action":"%s{action}","transactionsize":"%d{totalsize}","responsesize":"%d{respsize}","requestsize":"%d{reqsize}","urlcategory":"%s{urlcat}","serverip":"%s{sip}","requestmethod":"%s{reqmethod}","refererURL":"%s{ereferer}","useragent":"%s{eua}","product":"NSS","location":"%s{elocation}","ClientIP":"%s{cip}","status":"%s{respcode}","user":"%s{elogin}","url":"%s{eurl}","vendor":"Zscaler","hostname":"%s{ehost}","clientpublicIP":"%s{cintip}","threatcategory":"%s{malwarecat}","threatname":"%s{threatname}","filetype":"%s{filetype}","appname":"%s{appname}","app_status":"%s{app_status}","pagerisk":"%d{riskscore}","threatseverity":"%s{threatseverity}","department":"%s{edepartment}","urlsupercategory":"%s{urlsupercat}","appclass":"%s{appclass}","dlpengine":"%s{dlpeng}","urlclass":"%s{urlclass}","threatclass":"%s{malwareclass}","dlpdictionaries":"%s{dlpdict}","fileclass":"%s{fileclass}","bwthrottle":"%s{bwthrottle}","contenttype":"%s{contenttype}","unscannabletype":"%s{unscannabletype}","deviceowner":"%s{deviceowner}","devicehostname":"%s{devicehostname}","keyprotectiontype":"%s{keyprotectiontype}"\\}\\}\n',
                        "feedStatus": "ENABLED",
                        "id": 0,
                        "jsonArrayToggle": True,
                        "name": "test-feed-2",
                        "nssFeedType": "JSON",
                        "nssFilter": {"securityFeedFilter": False},
                        "nssLogType": "WEBLOG",
                        "siemConfiguration": {},
                        "timeZone": "GMT",
                    },
                    "recordid": "370",
                    "resource": "test-feed-2",
                    "result": "SUCCESS",
                    "subcategory": "NSS_FEED",
                    "time": "2024-11-04 16:33:48.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



"""
Global helpers for Sublime detections
"""


def sublime_alert_context(event) -> dict:
    context = {}
    context["events_type"] = event.get("type", "<TYPE_NOT_FOUND>")
    context["users_emails"] = event.deep_get(
        "created_by",
        "email_address",
        default="<EMAIL_NOT_FOUND>",
    )
    context["users_role"] = event.deep_get("created_by", "role", default="<ROLES_NOT_FOUND>")
    context["request_ip"] = event.deep_get("data", "request", "ip", default="<IP_NOT_FOUND>")
    context["request_path"] = event.deep_get("data", "request", "path", default="<PATH_NOT_FOUND>")
    return context



from pypanther.rules.aws_securityfinding.decoy_dynamodb_accessed import DecoyDynamoDBAccessed as DecoyDynamoDBAccessed
from pypanther.rules.aws_securityfinding.decoy_iam_assumed import DecoyIAMAssumed as DecoyIAMAssumed
from pypanther.rules.aws_securityfinding.decoy_s3_accessed import DecoyS3Accessed as DecoyS3Accessed
from pypanther.rules.aws_securityfinding.decoy_secret_accessed import DecoySecretAccessed as DecoySecretAccessed
from pypanther.rules.aws_securityfinding.decoy_systems_manager_parameter_accessed import (
    DecoySystemsManagerParameterAccessed as DecoySystemsManagerParameterAccessed,
)



"""
Used to define global helpers and variables.
"""

import re
from base64 import b64decode
from binascii import Error as AsciiError
from collections import OrderedDict
from collections.abc import Mapping
from datetime import datetime
from fnmatch import fnmatch
from functools import reduce
from ipaddress import ip_address, ip_network
from typing import Any, List, Optional, Sequence, Union

from dateutil import parser

# # # # # # # # # # # # # #
#       Exceptions        #
# # # # # # # # # # # # # #


class PantherUnexpectedAlert(Exception):
    """Error returned when a Panther detection encounters an unexpected situation."""


# # # # # # # # # # # # # #
#      Generic Helpers    #
# # # # # # # # # # # # # #

EMAIL_REGEX = re.compile(r"[\w.+%-]+@[\w.-]+\.[a-zA-Z]{2,}")


def deep_get(dictionary: dict, *keys, default=None):
    """
    Safely return the value of an arbitrarily nested map

    Inspired by https://bit.ly/3a0hq9E
    """
    out = reduce(
        lambda d, key: d.get(key, default) if isinstance(d, Mapping) else default,
        keys,
        dictionary,
    )
    if out is None:
        return default
    return out


# pylint: disable=too-complex,too-many-return-statements
def deep_walk(
    obj: Optional[Any],
    *keys: str,
    default: Optional[str] = None,
    return_val: str = "all",
) -> Union[Optional[Any], Optional[List[Any]]]:
    """
    Safely retrieve a value stored in complex dictionary structure

    Similar to deep_get but supports accessing dictionary keys within nested lists as well

    Parameters
    ----------
    obj (any): the original log event passed to rule(event)
               and nested objects retrieved recursively
    keys (str): comma-separated list of keys used to traverse the event object
    default (str): the default value to return if the desired key's value is not present
    return_val (str): string specifying which value to return
                      possible values are "first", "last", or "all"

    Returns
    -------
    any | list[any]: A single value if return_val is "first", "last",
                     or if "all" is a list containing one element,
                     otherwise a list of values

    """

    def _empty_list(sub_obj: Any):
        return (
            all(_empty_list(next_obj) for next_obj in sub_obj)
            if isinstance(sub_obj, Sequence) and not isinstance(sub_obj, str)
            else False
        )

    if not keys:
        return default if _empty_list(obj) else obj

    current_key = keys[0]
    found: OrderedDict = OrderedDict()

    if isinstance(obj, Mapping):
        next_key = obj.get(current_key, None)
        return (
            deep_walk(next_key, *keys[1:], default=default, return_val=return_val) if next_key is not None else default
        )
    if isinstance(obj, Sequence) and not isinstance(obj, str):
        for item in obj:
            value = deep_walk(item, *keys, default=default, return_val=return_val)
            if value is not None:
                if isinstance(value, Sequence) and not isinstance(value, str):
                    for sub_item in value:
                        found[sub_item] = None
                else:
                    found[value] = None

    found_list: list[Any] = list(found.keys())
    if not found_list:
        return default
    return {
        "first": found_list[0],
        "last": found_list[-1],
        "all": found_list[0] if len(found_list) == 1 else found_list,
    }.get(return_val, "all")


def get_val_from_list(list_of_dicts, return_field_key, field_cmp_key, field_cmp_val):
    """
    Return a specific field in a list of Python dictionaries.
    We return the empty set if the comparison key is not found
    """
    values_of_return_field = set()
    for item in list_of_dicts:
        if item.get(field_cmp_key) == field_cmp_val:
            values_of_return_field.add(item.get(return_field_key))
    return values_of_return_field


def is_ip_in_network(ip_addr, networks):
    """Check that a given IP is within a list of IP ranges"""
    return any(ip_address(ip_addr) in ip_network(network) for network in networks)


def pattern_match(string_to_match: str, pattern: str):
    """Wrapper around fnmatch for basic pattern globs"""
    return fnmatch(string_to_match, pattern)


def pattern_match_list(string_to_match: str, patterns: Sequence[str]):
    """Check that a string matches any pattern in a given list"""
    return any(fnmatch(string_to_match, p) for p in patterns)


def defang_ioc(ioc: str) -> str:
    """Return defanged IOC from 1.1.1.1 to 1[.]1[.]1[.]1"""
    ioc = ioc.replace("http://", "hxxp://")
    ioc = ioc.replace("https://", "hxxps://")
    return ioc.replace(".", "[.]")


# IOC Helper functions:
def ioc_match(indicators: list, known_iocs: set) -> list:
    """
    Matches a set of indicators against known Indicators of Compromise

    :param indicators: List of potential indicators of compromise
    :param known_iocs: Set of known indicators of compromise
    :return: List of any indicator matches
    """
    # Check through the IP IOCs
    return [ioc for ioc in (indicators or []) if ioc in known_iocs]


def panther_nanotime_to_python_datetime(panther_time: str) -> datetime:
    panther_time_micros = re.search(r"\.(\d+)", panther_time).group(1)
    panther_time_micros_rounded = panther_time_micros[0:6]
    panther_time_rounded = re.sub(r"\.\d+", f".{panther_time_micros_rounded}", panther_time)
    panther_time_format = r"%Y-%m-%d %H:%M:%S.%f"
    return datetime.strptime(panther_time_rounded, panther_time_format)


def golang_nanotime_to_python_datetime(golang_time: str) -> datetime:
    golang_time_format = r"%Y-%m-%dT%H:%M:%S.%fZ"
    # Golang fractional seconds include a mix of microseconds and
    # nanoseconds, which doesn't play well with Python's microseconds datetimes.
    # This rounds the fractional seconds to a microsecond-size.
    golang_time_micros = re.search(r"\.(\d+)Z", golang_time).group(1)
    golang_time_micros_rounded = golang_time_micros[0:6]
    golang_time_rounded = re.sub(r"\.\d+Z", f".{golang_time_micros_rounded}Z", golang_time)
    return datetime.strptime(golang_time_rounded, golang_time_format)


def is_base64(b64: str) -> str:
    # if the string is base64 encoded, return the decoded ASCII string
    # otherwise return an empty string
    # handle false positives for very short strings
    if len(b64) < 12:
        return ""
    # Pad args with "=" to ensure proper decoding
    b64 = b64.ljust((len(b64) + 3) // 4 * 4, "=")
    # Check if the matched string can be decoded back into ASCII
    try:
        return b64decode(b64, validate=True).decode("ascii")
    except AsciiError:
        pass
    except UnicodeDecodeError:
        pass
    return ""


def key_value_list_to_dict(list_objects: List[dict], key: str, value: str) -> dict:
    # Convert a list of dictionaries to a single dictionary
    # example: [{'key': 'a', 'value': 1}, {'key': 'b', 'value': 2}]
    # becomes: {'a': 1, 'b': 2}
    return {item[key]: item[value] for item in list_objects}


# When a single item is loaded from json, it is loaded as a single item
# When a list of items is loaded from json, it is loaded as a list of that item
# When we want to iterate over something that could be a single item or a list
# of items we can use listify and just continue as if it's always a list
def listify(maybe_list):
    try:
        iter(maybe_list)
    except TypeError:
        # not a list
        return [maybe_list]
    # either a list or string
    return [maybe_list] if isinstance(maybe_list, (str, bytes, dict)) else maybe_list


# Auto Time Resolution Parameters
EPOCH_REGEX = r"([0-9]{9,12}(\.\d+)?)"
TIME_FORMATS = [
    "%Y-%m-%d %H:%M:%S",  # Panther p_event_time Timestamp
    "%Y-%m-%dT%H:%M:%SZ",  # AWS Timestamp
    "%Y-%m-%dT%H:%M:%S.%fZ",  # Panther Timestamp
    "%Y-%m-%dT%H:%M:%S*%f%z",
    "%Y %b %d %H:%M:%S.%f %Z",
    "%b %d %H:%M:%S %z %Y",
    "%d/%b/%Y:%H:%M:%S %z",
    "%b %d, %Y %I:%M:%S %p",
    "%b %d %Y %H:%M:%S",
    "%b %d %H:%M:%S %Y",
    "%b %d %H:%M:%S %z",
    "%b %d %H:%M:%S",
    "%Y-%m-%dT%H:%M:%S%z",
    "%Y-%m-%dT%H:%M:%S.%f%z",
    "%Y-%m-%d %H:%M:%S %z",
    "%Y-%m-%d %H:%M:%S%z",
    "%Y-%m-%d %H:%M:%S,%f",
    "%Y/%m/%d*%H:%M:%S",
    "%Y %b %d %H:%M:%S.%f*%Z",
    "%Y %b %d %H:%M:%S.%f",
    "%Y-%m-%d %H:%M:%S,%f%z",
    "%Y-%m-%d %H:%M:%S.%f",
    "%Y-%m-%d %H:%M:%S.%f%z",
    "%Y-%m-%dT%H:%M:%S.%f",
    "%Y-%m-%dT%H:%M:%S",
    "%Y-%m-%dT%H:%M:%S%z",
    "%Y-%m-%dT%H:%M:%S.%f",
    "%Y-%m-%dT%H:%M:%S",
    "%Y-%m-%d*%H:%M:%S:%f",
    "%Y-%m-%d*%H:%M:%S",
    "%y-%m-%d %H:%M:%S,%f %z",
    "%y-%m-%d %H:%M:%S,%f",
    "%y-%m-%d %H:%M:%S",
    "%y/%m/%d %H:%M:%S",
    "%y%m%d %H:%M:%S",
    "%Y%m%d %H:%M:%S.%f",
    "%m/%d/%y*%H:%M:%S",
    "%m/%d/%Y*%H:%M:%S",
    "%m/%d/%Y*%H:%M:%S*%f",
    "%m/%d/%y %H:%M:%S %z",
    "%m/%d/%Y %H:%M:%S %z",
    "%H:%M:%S",
    "%H:%M:%S.%f",
    "%H:%M:%S,%f",
    "%d/%b %H:%M:%S,%f",
    "%d/%b/%Y:%H:%M:%S",
    "%d/%b/%Y %H:%M:%S",
    "%d-%b-%Y %H:%M:%S",
    "%d-%b-%Y %H:%M:%S.%f",
    "%d %b %Y %H:%M:%S",
    "%d %b %Y %H:%M:%S*%f",
    "%m%d_%H:%M:%S",
    "%m%d_%H:%M:%S.%f",
    "%m/%d/%Y %I:%M:%S %p:%f",
    "%m/%d/%Y %I:%M:%S %p",
]


def resolve_timestamp_string(timestamp: str) -> Optional[datetime]:
    """Auto Time Resolution"""
    if not timestamp:
        return None

    # Removes weird single-quotes used in some timestamp formats
    ts_format = timestamp.replace("'", "")
    # Attempt to resolve timestamp format
    for each_format in TIME_FORMATS:
        try:
            return datetime.strptime(ts_format, each_format)
        except (ValueError, TypeError):
            continue
    try:
        return parser.parse(timestamp)
    except (ValueError, TypeError, parser.ParserError):
        pass

    # Attempt to resolve epoch format
    # Since datetime.utcfromtimestamp supports 9 through 12 digit epoch timestamps
    # and we only want the first 12 digits.
    match = re.match(EPOCH_REGEX, timestamp)
    if match.group(0) != "":
        try:
            return datetime.utcfromtimestamp(float(match.group(0)))
        except (ValueError, TypeError):
            return None
    return None


# returns the difference between time1 and later time 2 in human-readable time period string
def time_delta(time1, time2: str) -> str:
    time1_truncated = nano_to_micro(time1)
    time2_truncated = nano_to_micro(time2)
    delta_timedelta = resolve_timestamp_string(time2_truncated) - resolve_timestamp_string(
        time1_truncated,
    )
    days = delta_timedelta.days
    hours, remainder = divmod(delta_timedelta.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    delta = ""
    if days > 0:
        delta = f"{days} day(s) "
    if hours > 0:
        delta = "".join([delta, f"{hours} hour(s) "])
    if minutes > 0:
        delta = "".join([delta, f"{minutes} minute(s) "])
    if seconds > 0:
        delta = "".join([delta, f"{seconds} second(s)"])
    return delta


def nano_to_micro(time_str: str) -> str:
    parts = time_str.split(":")
    # pylint: disable=consider-using-f-string
    parts[-1] = f"{float(parts[-1]):06f}"
    return ":".join(parts)


# adds parsing delay to an alert_context
def add_parse_delay(event, context: dict) -> dict:
    parsing_delay = time_delta(event.get("p_event_time"), event.get("p_parse_time"))
    context["parseDelay"] = f"{parsing_delay}"
    return context


# generate a PantherFlow investigation from an event
def pantherflow_investigation(event, interval="30m"):
    logtype = event.get("p_log_type", "").lower().replace(".", "_")
    timestamp = event.get("p_event_time", "")

    query = f"""union panther_signals.public.correlation_signals
    , panther_logs.public.{logtype}
| where p_event_time between time.parse_timestamp('{timestamp}') - time.parse_timespan('{interval}') .. time.parse_timestamp('{timestamp}') + time.parse_timespan('{interval}')
"""

    first = True
    for key, value in event.items():
        if key.startswith("p_any_") and key != "p_any_aws_account_ids":
            if first:
                query += f"| where arrays.overlap({key}, {value.copy()})\n"
                first = False
            else:
                query += f"     or arrays.overlap({key}, {value.copy()})\n"
    query += "| sort p_event_time"

    return query



import inspect
import json
from typing import Any, Callable, Dict, Type

from panther_core.enriched_event import PantherEvent
from prettytable import PrettyTable

from pypanther import utils
from pypanther.base import RULE_ALL_METHODS, Rule

DEFAULT_RULE_TABLE_ATTRS = [
    "id",
    "log_types",
    "default_severity",
    "enabled",
]

DEFAULT_RULE_TABLE_SORT_BY = "id"

ALL_TABLE_ATTR = "all"
VALID_RULE_TABLE_ATTRS = [
    *DEFAULT_RULE_TABLE_ATTRS,
    "create_alert",
    "dedup_period_minutes",
    "display_name",
    "summary_attributes",
    "threshold",
    "tags",
    "default_description",
    "default_reference",
    "default_runbook",
    "default_destinations",
]

OUTPUT_TYPE_JSON = "json"
OUTPUT_TYPE_TEXT = "text"
OUTPUT_TYPE_CSV = "csv"
DEFAULT_CLI_OUTPUT_TYPE = OUTPUT_TYPE_TEXT
COMMON_CLI_OUTPUT_TYPES = [
    OUTPUT_TYPE_TEXT,
    OUTPUT_TYPE_JSON,
]

JSON_INDENT_LEVEL = 2


def print_rule_table(
    rules: list[Type[Rule]],
    attributes: list[str] | None = None,
    print_total: bool = True,
    sort_by: str | None = None,
) -> None:
    """
    Prints rules in a table format for easy viewing.

    Parameters
    ----------
        rules (list[Type[Rule]]): The list of PantherRule subclasses that will be printed in table format.
        attributes (list[str] | None): The list of attributes that will appear as columns in the table.
            Supplying None or an empty list will use defaults of [id, log_types, default_severity, enabled].
        print_total (bool): Whether to print the total number of rules at the end.
        sort_by (str | None): The attribute to sort the rules by. If None, defaults to 'id' or first attribute.

    """
    attributes = utils.dedup_list_preserving_order(attributes or [])
    check_rule_attributes(attributes)

    if not attributes:
        attributes = DEFAULT_RULE_TABLE_ATTRS

    if not sort_by or sort_by not in attributes:
        sort_by = "id" if "id" in attributes else attributes[0]

    rule_dicts = sorted(
        [{attr: getattr(rule, attr) for attr in attributes} for rule in rules],
        key=lambda d: d[sort_by],
    )

    table = PrettyTable(field_names=attributes)
    # Set max width for columns to prevent table from becoming too wide
    for field in attributes:
        table.max_width[field] = 50
    table.add_rows(
        [[pretty_format_text(value) for value in rule_dict.values()] for rule_dict in rule_dicts],
    )

    print(table)
    if print_total:
        print(f"Total rules: {len(rules)}")


def pretty_format_text(val) -> str:
    if val == "" or val is None or val == []:
        return "-"

    if isinstance(val, list):
        if len(val) > 2:
            val = val[:2] + [f"+{len(val) - 2}"]

        return ", ".join([str(s) for s in val])

    return val


def print_rules_as_json(
    rules: list[Type[Rule]],
    attributes: list[str] | None = None,
    sort_by: str | None = None,
) -> None:
    """
    Prints rules in JSON format for easy viewing.

    Parameters
    ----------
        rules (list[Type[Rule]]): The list of PantherRule subclasses that will be printed in JSON format.
        attributes (list[str] | None): The list of attributes that will appear as attributes in the JSON.
            Supplying None or an empty list will use defaults of [id, log_types, default_severity, enabled].

    """
    attributes = utils.dedup_list_preserving_order(attributes or [])
    check_rule_attributes(attributes)

    if not attributes:
        attributes = DEFAULT_RULE_TABLE_ATTRS

    if not sort_by or sort_by not in attributes:
        sort_by = "id" if "id" in attributes else attributes[0]

    rule_dicts = sorted(
        [{attr: getattr(rule, attr) for attr in attributes} for rule in rules],
        key=lambda d: d[sort_by],
    )

    print(json.dumps({"rules": rule_dicts, "total_rules": len(rule_dicts)}, indent=JSON_INDENT_LEVEL))


def print_rules_as_csv(
    rules: list[Type[Rule]],
    attributes: list[str] | None = None,
    sort_by: str | None = None,
) -> None:
    """
    Prints rules in CSV format for easy viewing and parsing.

    Parameters
    ----------
        rules (list[Type[Rule]]): The list of PantherRule subclasses that will be printed in CSV format.
        attributes (list[str] | None): The list of attributes that will appear as attributes in the CSV.
            Supplying None or an empty list will use defaults of [id, log_types, default_severity, enabled].

    """
    attributes = utils.dedup_list_preserving_order(attributes or [])
    check_rule_attributes(attributes)

    if not attributes:
        attributes = DEFAULT_RULE_TABLE_ATTRS

    if not sort_by or sort_by not in attributes:
        sort_by = "id" if "id" in attributes else attributes[0]

    rule_dicts = sorted(
        [{attr: getattr(rule, attr) for attr in attributes} for rule in rules],
        key=lambda d: d[sort_by],
    )

    # print the column labels as the header row
    print(",".join(attributes))

    # print the data rows
    for rule_dict in rule_dicts:
        print(",".join([pretty_format_csv(rule_dict[k]) for k in rule_dict]))


def pretty_format_csv(value: Any) -> str:
    # take care of lists by quoting them
    if isinstance(value, list):
        return f'"{",".join([str(val) for val in value])}"'

    return str(value)


def check_rule_attributes(attributes: list[str]) -> None:
    if diff := set(attributes) - set(VALID_RULE_TABLE_ATTRS):
        raise AttributeError(f"Attributes '{list(diff)}' is not allowed.")


def _get_rule_dict_base(rule: Type[Rule]) -> Dict[str, Any]:
    rule_dict = rule.asdict()
    del rule_dict["tests"]
    rule_dict["include_filters"] = _prettify_filters(rule_dict["include_filters"])
    rule_dict["exclude_filters"] = _prettify_filters(rule_dict["exclude_filters"])
    for method in RULE_ALL_METHODS:
        method_attr = getattr(rule, method)
        try:
            rule_dict[method] = "\n" + inspect.getsource(method_attr)
        except BaseException:
            rule_dict[method] = repr(method_attr)
    return rule_dict


def _prettify_filters(filters: list[Callable[[PantherEvent], bool]]) -> list[str]:
    pretty_filters = []
    for filter_ in filters:
        pretty_filters.append(filter_.__name__)
    return pretty_filters


def print_rule_as_json(rule: Type[Rule], managed: bool) -> None:
    rule_dict = {}
    if managed:
        source = inspect.getsource(rule)
        rule_dict["class_definition"] = source
    else:
        rule_dict = _get_rule_dict_base(rule)
    rule_json = json.dumps(rule_dict, indent=JSON_INDENT_LEVEL)
    print(rule_json)


def print_rule_as_text(rule: Type[Rule], original: bool) -> None:
    if original:
        rule_text = inspect.getsource(rule)
    else:
        rule_text = f"class {rule.__name__}:\n"
        rule_dict = _get_rule_dict_base(rule)
        for k, v in rule_dict.items():
            rule_text += f"    {k} = {v}\n"
    print(rule_text)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context, lookup_aws_account_name


@panther_managed
class AWSCloudTrailEventSelectorsDisabled(Rule):
    id = "AWS.CloudTrail.EventSelectorsDisabled-prototype"
    display_name = "CloudTrail Event Selectors Disabled"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["3.5"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.MEDIUM
    default_description = "A CloudTrail Trail was modified to exclude management events for 1 or more resource types.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-cloudtrail-modified"
    default_reference = (
        "https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-update-a-trail-console.html"
    )
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    # API calls that are indicative of CloudTrail changes
    CLOUDTRAIL_EDIT_SELECTORS = {"PutEventSelectors"}

    def rule(self, event):
        if not (aws_cloudtrail_success(event) and event.get("eventName") in self.CLOUDTRAIL_EDIT_SELECTORS):
            return False
        # Check if management events are included for each selector.
        #    deep_walk only returns a list if there's more than 1 entry in the nested array, so we must
        #    enforce it to be a list.
        includes = event.deep_walk("requestParameters", "eventSelectors", "includeManagementEvents")
        if not isinstance(includes, list):
            includes = [includes]
        # Return False all the management events are included, else return True and raise alert
        return not all(includes)

    def dedup(self, event):
        # Merge on the CloudTrail ARN
        return event.deep_get("requestParameters", "trailName", default="<UNKNOWN_NAME>")

    def title(self, event):
        return f"Management events have been exluded from CloudTrail [{self.dedup(event)}] in account [{lookup_aws_account_name(event.get('recipientAccountId'))}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Event Selector Disabled",
            expected_result=True,
            log={
                "p_event_time": "2024-11-25 17:51:21.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-25 17:55:54.253083422",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "4ca1cb25-7633-496b-8f92-6de876228c3f",
                "eventName": "PutEventSelectors",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventTime": "2024-11-25 17:51:21.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.11",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "a8c6184a-89b1-4fc1-a6fa-324748d48b64",
                "requestParameters": {
                    "eventSelectors": [
                        {
                            "dataResources": [
                                {"type": "AWS::S3::Object", "values": []},
                                {"type": "AWS::Lambda::Function", "values": []},
                            ],
                            "excludeManagementEventSources": [],
                            "includeManagementEvents": False,
                            "readWriteType": "ReadOnly",
                        },
                    ],
                    "trailName": "sample-cloudtrail-name",
                },
                "responseElements": {
                    "eventSelectors": [
                        {
                            "dataResources": [
                                {"type": "AWS::S3::Object", "values": []},
                                {"type": "AWS::Lambda::Function", "values": []},
                            ],
                            "excludeManagementEventSources": [],
                            "includeManagementEvents": False,
                            "readWriteType": "ReadOnly",
                        },
                    ],
                    "trailARN": "arn:aws:cloudtrail:us-west-2:111122223333:trail/sample-cloudtrail-name",
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "cloudtrail.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY_ID",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "EXAMPLEPRINCIPLEID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-25T16:53:42Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "EXAMPLEPRINCIPLEID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Event Selector Enabled",
            expected_result=False,
            log={
                "p_event_time": "2024-11-25 17:51:21.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-25 17:55:54.253083422",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "4ca1cb25-7633-496b-8f92-6de876228c3f",
                "eventName": "PutEventSelectors",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventTime": "2024-11-25 17:51:21.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.11",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "a8c6184a-89b1-4fc1-a6fa-324748d48b64",
                "requestParameters": {
                    "eventSelectors": [
                        {
                            "dataResources": [],
                            "excludeManagementEventSources": [],
                            "includeManagementEvents": True,
                            "readWriteType": "All",
                        },
                    ],
                    "trailName": "sample-cloudtrail-name",
                },
                "responseElements": {
                    "eventSelectors": [
                        {
                            "dataResources": [],
                            "excludeManagementEventSources": [],
                            "includeManagementEvents": True,
                            "readWriteType": "All",
                        },
                    ],
                    "trailARN": "arn:aws:cloudtrail:us-west-2:111122223333:trail/sample-cloudtrail-name",
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "cloudtrail.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY_ID",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "EXAMPLEPRINCIPLEID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-25T16:53:42Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "EXAMPLEPRINCIPLEID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Uninteresting Event Type",
            expected_result=False,
            log={
                "p_event_time": "2024-11-25 17:50:24.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-25 17:55:54.172592534",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "63fb143a-c494-4510-8e9e-34172e4872c3",
                "eventName": "GetEventSelectors",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventTime": "2024-11-25 17:50:24.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.11",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "cad6aff4-1558-49c5-ae4a-c512058751f1",
                "requestParameters": {"trailName": "sample-cloudtrail-name"},
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "cloudtrail.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "APN/1.0 HashiCorp/1.0 Terraform/1.1.2 (+https://www.terraform.io) terraform-provider-aws/3.76.1 (+https://registry.terraform.io/providers/hashicorp/aws) aws-sdk-go/1.44.157 (go1.19.3; darwin; arm64) stratus-red-team_83c9a458-ffab-4d43-8b02-9691311e8c0a HashiCorp-terraform-exec/0.17.3",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY_ID",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "EXAMPLEPRINCIPLEID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-25T16:53:42Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "EXAMPLEPRINCIPLEID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context, lookup_aws_account_name


@panther_managed
class AWSCloudTrailStopped(Rule):
    id = "AWS.CloudTrail.Stopped-prototype"
    display_name = "CloudTrail Stopped"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "DemoThreatHunting", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["3.5"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.MEDIUM
    default_description = "A CloudTrail Trail was modified.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-cloudtrail-modified"
    default_reference = (
        "https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-delete-trails-console.html"
    )
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    # API calls that are indicative of CloudTrail changes
    CLOUDTRAIL_STOP_DELETE = {"DeleteTrail", "StopLogging"}

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.CLOUDTRAIL_STOP_DELETE

    def dedup(self, event):
        # Merge on the CloudTrail ARN
        return event.deep_get("requestParameters", "name", default="<UNKNOWN_NAME>")

    def title(self, event):
        return f"CloudTrail [{self.dedup(event)}] in account [{lookup_aws_account_name(event.get('recipientAccountId'))}] was stopped/deleted"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="CloudTrail Was Stopped",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventName": "StopLogging",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"name": "arn:aws:cloudtrail:us-west-2:123456789012:trail/example-trail"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="CloudTrail Was Started",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "111:panther-snapshot-scheduler",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventName": "StartLogging",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "encryptionContext": {
                        "aws:lambda:FunctionArn": "arn:aws:lambda:us-west-2:123456789012:function:test-function",
                    },
                },
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": True,
                "resources": [
                    {
                        "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                        "accountId": "123456789012",
                        "type": "AWS::KMS::Key",
                    },
                ],
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Error Stopping CloudTrail",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "errorCode": "InvalidTrailNameException",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventName": "StopLogging",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"name": "arn:aws:cloudtrail:us-west-2:123456789012:trail/example-trail"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class ThinkstCanaryDCRC(Rule):
    display_name = "Thinkst Canary DCRC"
    id = "Thinkst.CanaryDCRC-prototype"
    default_description = "A Canary has disconnected/reconnected."
    default_severity = Severity.HIGH
    log_types = [LogType.THINKSTCANARY_ALERT]

    def rule(self, event):
        return any(keyword in event.get("Intro", "") for keyword in ["disconnected", "reconnected"])

    def title(self, event):
        return event.get("Intro", "Canary Disconnected/Reconnected")

    def severity(self, event):
        if "reconnected" in event.get("Intro", ""):
            return "Low"
        return "Default"

    tests = [
        RuleTest(
            name="Canary Disconnected",
            expected_result=True,
            log={
                "CanaryID": "00029666d14d454f",
                "CanaryIP": "192.168.20.101",
                "CanaryName": "FS01",
                "Description": "Canary Disconnected",
                "IncidentKey": "incident:devicedied:3b04b62c54dcbb64d17131be::1718794923",
                "Intro": "One of your Canaries (FS01) previously at 192.168.20.101 has disconnected.",
                "MatchedAnnotations": {},
                "Timestamp": "2024-06-19 11:02:03 (UTC)",
            },
        ),
    ]



from pypanther.rules.github.github_action_failed import GitHubActionFailed as GitHubActionFailed
from pypanther.rules.github.github_advanced_security_change import (
    GitHubAdvancedSecurityChange as GitHubAdvancedSecurityChange,
)
from pypanther.rules.github.github_branch_policy_override import (
    GitHubBranchPolicyOverride as GitHubBranchPolicyOverride,
)
from pypanther.rules.github.github_branch_protection_disabled import (
    GitHubBranchProtectionDisabled as GitHubBranchProtectionDisabled,
)
from pypanther.rules.github.github_org_auth_modified import GitHubOrgAuthChange as GitHubOrgAuthChange
from pypanther.rules.github.github_org_ip_allowlist import GitHubOrgIpAllowlist as GitHubOrgIpAllowlist
from pypanther.rules.github.github_org_moderators_add import GitHubOrgModeratorsAdd as GitHubOrgModeratorsAdd
from pypanther.rules.github.github_org_modified import GitHubOrgModified as GitHubOrgModified
from pypanther.rules.github.github_organization_app_integration_installed import (
    GithubOrganizationAppIntegrationInstalled as GithubOrganizationAppIntegrationInstalled,
)
from pypanther.rules.github.github_public_repository_created import (
    GithubPublicRepositoryCreated as GithubPublicRepositoryCreated,
)
from pypanther.rules.github.github_repo_collaborator_change import (
    GithubRepoCollaboratorChange as GithubRepoCollaboratorChange,
)
from pypanther.rules.github.github_repo_created import GithubRepoCreated as GithubRepoCreated
from pypanther.rules.github.github_repo_initial_access import GitHubRepoInitialAccess as GitHubRepoInitialAccess
from pypanther.rules.github.github_repo_ruleset_modified import GitHubRepoRulesetModified as GitHubRepoRulesetModified
from pypanther.rules.github.github_repo_visibility_change import (
    GithubRepoVisibilityChange as GithubRepoVisibilityChange,
)
from pypanther.rules.github.github_repo_vulnerability_dismissed import (
    GithubRepoVulnerabilityDismissed as GithubRepoVulnerabilityDismissed,
)
from pypanther.rules.github.github_repository_transfer import GithubRepositoryTransfer as GithubRepositoryTransfer
from pypanther.rules.github.github_secret_scanning_alert_created import (
    GitHubSecretScanningAlertCreated as GitHubSecretScanningAlertCreated,
)
from pypanther.rules.github.github_team_modified import GitHubTeamModified as GitHubTeamModified
from pypanther.rules.github.github_user_access_key_created import (
    GitHubUserAccessKeyCreated as GitHubUserAccessKeyCreated,
)
from pypanther.rules.github.github_user_role_updated import GitHubUserRoleUpdated as GitHubUserRoleUpdated
from pypanther.rules.github.github_webhook_modified import GitHubWebhookModified as GitHubWebhookModified



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteTwoStepVerification(Rule):
    id = "GSuite.TwoStepVerification-prototype"
    display_name = "GSuite User Two Step Verification Change"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_severity = Severity.LOW
    default_description = "A user disabled two step verification for themselves.\n"
    default_reference = (
        "https://support.google.com/mail/answer/185839?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    )
    default_runbook = (
        "Depending on company policy, either suggest or require the user re-enable two step verification.\n"
    )
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "user_accounts":
            return False
        if event.get("type") == "2sv_change" and event.get("name") == "2sv_disable":
            return True
        return False

    def title(self, event):
        return f"Two step verification was disabled for user [{event.deep_get('actor', 'email', default='<UNKNOWN_USER>')}]"

    tests = [
        RuleTest(
            name="Two Step Verification Enabled",
            expected_result=False,
            log={
                "id": {"applicationName": "user_accounts"},
                "actor": {"callerType": "USER", "email": "some.user@somedomain.com"},
                "kind": "admin#reports#activity",
                "type": "2sv_change",
                "name": "2sv_enroll",
            },
        ),
        RuleTest(
            name="Two Step Verification Disabled",
            expected_result=True,
            log={
                "id": {"applicationName": "user_accounts"},
                "actor": {"callerType": "USER", "email": "some.user@somedomain.com"},
                "kind": "admin#reports#activity",
                "type": "2sv_change",
                "name": "2sv_disable",
            },
        ),
    ]



from pypanther.rules.asana.asana_service_account_created import AsanaServiceAccountCreated as AsanaServiceAccountCreated
from pypanther.rules.asana.asana_team_privacy_public import AsanaTeamPrivacyPublic as AsanaTeamPrivacyPublic
from pypanther.rules.asana.asana_workspace_default_session_duration_never import (
    AsanaWorkspaceDefaultSessionDurationNever as AsanaWorkspaceDefaultSessionDurationNever,
)
from pypanther.rules.asana.asana_workspace_email_domain_added import (
    AsanaWorkspaceEmailDomainAdded as AsanaWorkspaceEmailDomainAdded,
)
from pypanther.rules.asana.asana_workspace_form_link_auth_requirement_disabled import (
    AsanaWorkspaceFormLinkAuthRequirementDisabled as AsanaWorkspaceFormLinkAuthRequirementDisabled,
)
from pypanther.rules.asana.asana_workspace_guest_invite_permissions_anyone import (
    AsanaWorkspaceGuestInvitePermissionsAnyone as AsanaWorkspaceGuestInvitePermissionsAnyone,
)
from pypanther.rules.asana.asana_workspace_new_admin import AsanaWorkspaceNewAdmin as AsanaWorkspaceNewAdmin
from pypanther.rules.asana.asana_workspace_org_export import AsanaWorkspaceOrgExport as AsanaWorkspaceOrgExport
from pypanther.rules.asana.asana_workspace_password_requirements_simple import (
    AsanaWorkspacePasswordRequirementsSimple as AsanaWorkspacePasswordRequirementsSimple,
)
from pypanther.rules.asana.asana_workspace_require_app_approvals_disabled import (
    AsanaWorkspaceRequireAppApprovalsDisabled as AsanaWorkspaceRequireAppApprovalsDisabled,
)
from pypanther.rules.asana.asana_workspace_saml_optional import AsanaWorkspaceSAMLOptional as AsanaWorkspaceSAMLOptional



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSSecretsManagerRetrieveSecrets(Rule):
    id = "AWS.SecretsManager.RetrieveSecrets-prototype"
    display_name = "EC2 Secrets Manager Retrieve Secrets"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Credential Access", "Stratus Red Team"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.MEDIUM
    default_description = "An attacker attempted to retrieve a high number of Secrets Manager secrets, through secretsmanager:GetSecretValue."
    default_runbook = "https://permiso.io/blog/lucr-3-scattered-spider-getting-saas-y-in-the-cloud"
    default_reference = (
        "https://stratus-red-team.cloud/attack-techniques/AWS/aws.credential-access.secretsmanager-retrieve-secrets/"
    )
    threshold = 20
    dedup_period_minutes = 1440
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        if (
            event.get("eventName") == "GetSecretValue"
            and (not aws_cloudtrail_success(event))
            and (event.get("errorCode") == "AccessDenied")
        ):
            return True
        return False

    def title(self, event):
        user = event.udm("actor_user")
        return f"[{user}] attempted to retrieve a large number of secrets from AWS Secrets Manager"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="GetSecretValue Denied",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "dfd6d93a-2ce6-4dbe-8939-86b8ba67c868",
                "eventName": "GetSecretValue",
                "errorCode": "AccessDenied",
                "eventSource": "secretsmanager.amazonaws.com",
                "eventTime": "2024-10-17 21:48:45.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.09",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "123123123123",
                "requestID": "5128b35f-0daf-4c5e-948a-21a1c507968c",
                "requestParameters": {
                    "secretId": "arn:aws:secretsmanager:us-west-2:123123123123:secret:stratus-red-team-retrieve-secret-3-gscOm8",
                    "versionId": "7DC59E8B-63AE-454D-B7A4-8A7D64AB05E7",
                },
                "sourceIPAddress": "123.123.123.123",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "secretsmanager.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "APN/1.0 HashiCorp/1.0 Terraform/1.1.2 (+https://www.terraform.io) terraform-provider-aws/3.76.1 (+https://registry.terraform.io/providers/hashicorp/aws) aws-sdk-go/1.44.157 (go1.19.3; darwin; arm64) HashiCorp-terraform-exec/0.17.3",
                "userIdentity": {
                    "accessKeyId": "ASIASXP6SDP2LKLKYYC4",
                    "accountId": "123123123123",
                    "arn": "arn:aws:sts::123123123123:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/evil.genius",
                    "principalId": "AROASXP6SDP2F4WLQVARB:evil.genius",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-10-17T21:48:13Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123123123123",
                            "arn": "arn:aws:iam::123123123123:role/aws-reserved/sso.amazonaws.com/us-west-2/AWSReservedSSO_DevAdmin_635426549a280cc6",
                            "principalId": "AROASXP6SDP2F4WLQVARB",
                            "type": "Role",
                            "userName": "AWSReservedSSO_DevAdmin_635426549a280cc6",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSCloudTrailLoginProfileCreatedOrModified(Rule):
    id = "AWS.CloudTrail.LoginProfileCreatedOrModified-prototype"
    display_name = "AWS User Login Profile Created or Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.LOW
    reports = {"MITRE ATT&CK": ["TA0003:T1098", "TA0005:T1108", "TA0005:T1550", "TA0008:T1550"]}
    default_description = "An attacker with iam:UpdateLoginProfile permission on other users can change the password used to login to the AWS console. May be legitimate account administration."
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_examples_aws_my-sec-creds-self-manage-pass-accesskeys-ssh.html"
    PROFILE_EVENTS = {"UpdateLoginProfile", "CreateLoginProfile", "DeleteLoginProfile"}

    def rule(self, event):
        # Only look for successes
        if not aws_cloudtrail_success(event):
            return False
        # Check when someone other than the user themselves creates or modifies a login profile
        # with no password reset needed
        return (
            event.get("eventSource", "") == "iam.amazonaws.com"
            and event.get("eventName", "") in self.PROFILE_EVENTS
            and (not event.deep_get("requestParameters", "passwordResetRequired", default=False))
            and (
                not event.deep_get("userIdentity", "arn", default="").endswith(
                    f"/{event.deep_get('requestParameters', 'userName', default='')}",
                )
            )
        )

    def title(self, event):
        return f"[{event.deep_get('userIdentity', 'arn')}] changed the password for [{event.deep_get('requestParameters', 'userName')}]"

    def alert_context(self, event):
        context = aws_rule_context(event)
        context["ip_and_username"] = event.get("sourceIPAddress", "<MISSING_SOURCE_IP>") + event.deep_get(
            "requestParameters",
            "userName",
            default="<MISSING_USER_NAME>",
        )
        return context

    tests = [
        RuleTest(
            name="ChangeOwnPassword",
            expected_result=False,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "1234",
                "eventName": "UpdateLoginProfile",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2022-09-15 13:45:24",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "987654321",
                "requestParameters": {"passwordResetRequired": False, "userName": "alice"},
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ABC1234",
                    "accountId": "987654321",
                    "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                    "principalId": "ABCDE:alice",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-15T13:36:47Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "987654321",
                            "arn": "arn:aws:iam::9876432:role/IAM",
                            "principalId": "1234ABC",
                            "type": "Role",
                            "userName": "IAM",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="User changed password for other",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "1234",
                "eventName": "UpdateLoginProfile",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2022-09-15 13:45:24",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "987654321",
                "requestParameters": {"passwordResetRequired": False, "userName": "bob"},
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ABC1234",
                    "accountId": "987654321",
                    "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                    "principalId": "ABCDE:alice",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-15T13:36:47Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "987654321",
                            "arn": "arn:aws:iam::9876432:role/IAM",
                            "principalId": "1234ABC",
                            "type": "Role",
                            "userName": "IAM",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.thinkstcanary import additional_details


@panther_managed
class ThinkstCanaryIncident(Rule):
    display_name = "Thinkst Canary Incident"
    id = "Thinkst.CanaryIncident-prototype"
    default_description = "A Canary incident has been detected."
    default_severity = Severity.HIGH
    log_types = [LogType.THINKSTCANARY_ALERT]

    def rule(self, event):
        return event.get("AlertType") == "CanaryIncident"

    def title(self, event):
        return event.get("Intro", "Canary Incident")

    def alert_context(self, event):
        return additional_details(event)

    tests = [
        RuleTest(
            name="Canary Incident",
            expected_result=True,
            log={
                "AdditionalDetails": [
                    ["User", "guest"],
                    ["Filename", "IT/Default Windows Desktop Configuration.docx"],
                    ["Background Context", "You have had 2 incidents from 192.168.110.14 previously."],
                ],
                "AlertType": "CanaryIncident",
                "CanaryID": "000222326791e1e8",
                "CanaryIP": "192.168.110.27",
                "CanaryLocation": "Server room A",
                "CanaryName": "VirtualCanary-unnamed",
                "CanaryPort": 445,
                "Description": "Shared File Opened",
                "IncidentHash": "f78b692a7716d0d668012bc0eb65c367",
                "IncidentKey": "incident:smbfileopen:89d38322e4e764e202b42bbb:192.168.110.14:1717059335",
                "Intro": "Shared File Opened has been detected against one of your Canaries (VirtualCanary-unnamed) at 192.168.110.27.",
                "ReverseDNS": "",
                "SourceIP": "192.168.110.14",
                "Timestamp": "2024-05-30 08:55:35 (UTC)",
            },
        ),
    ]



from pypanther.rules.sublime.sublime_mailboxes_deactivated import SublimeMailboxDeactivated as SublimeMailboxDeactivated
from pypanther.rules.sublime.sublime_message_flagged import SublimeMessageFlagged as SublimeMessageFlagged
from pypanther.rules.sublime.sublime_message_source_deleted_or_deactivated import (
    SublimeMessageSourceDeletedOrDeactivated as SublimeMessageSourceDeletedOrDeactivated,
)
from pypanther.rules.sublime.sublime_rules_deleted_or_deactivated import (
    SublimeRulesDeletedOrDeactivated as SublimeRulesDeletedOrDeactivated,
)



from pypanther.rules.wiz.wiz_alert_passthrough import WizAlertPassthrough as WizAlertPassthrough
from pypanther.rules.wiz.wiz_cicd_scan_policy_updated_or_deleted import (
    WizCICDScanPolicyUpdatedOrDeleted as WizCICDScanPolicyUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_connector_updated_or_deleted import (
    WizConnectorUpdatedOrDeleted as WizConnectorUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_data_classifier_updated_or_deleted import (
    WizDataClassifierUpdatedOrDeleted as WizDataClassifierUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_image_integrity_validator_updated_or_deleted import (
    WizImageIntegrityValidatorUpdatedOrDeleted as WizImageIntegrityValidatorUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_integration_updated_or_deleted import (
    WizIntegrationUpdatedOrDeleted as WizIntegrationUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_revoke_user_sessions import WizRevokeUserSessions as WizRevokeUserSessions
from pypanther.rules.wiz.wiz_rotate_service_account_secret import (
    WizRotateServiceAccountSecret as WizRotateServiceAccountSecret,
)
from pypanther.rules.wiz.wiz_rule_change import WizRuleChange as WizRuleChange
from pypanther.rules.wiz.wiz_saml_identity_provider_change import (
    WizSAMLIdentityProviderChange as WizSAMLIdentityProviderChange,
)
from pypanther.rules.wiz.wiz_service_account_change import WizServiceAccountChange as WizServiceAccountChange
from pypanther.rules.wiz.wiz_update_ip_restrictions import WizUpdateIPRestrictions as WizUpdateIPRestrictions
from pypanther.rules.wiz.wiz_update_login_settings import WizUpdateLoginSettings as WizUpdateLoginSettings
from pypanther.rules.wiz.wiz_update_scanner_settings import WizUpdateScannerSettings as WizUpdateScannerSettings
from pypanther.rules.wiz.wiz_update_support_contact_list import (
    WizUpdateSupportContactList as WizUpdateSupportContactList,
)
from pypanther.rules.wiz.wiz_user_created_or_deleted import WizUserCreatedOrDeleted as WizUserCreatedOrDeleted
from pypanther.rules.wiz.wiz_user_role_updated_or_deleted import (
    WizUserRoleUpdatedOrDeleted as WizUserRoleUpdatedOrDeleted,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteDeviceSuspiciousActivity(Rule):
    id = "GSuite.DeviceSuspiciousActivity-prototype"
    display_name = "GSuite Device Suspicious Activity"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.LOW
    default_description = "GSuite reported a suspicious activity on a user's device.\n"
    default_reference = "https://support.google.com/a/answer/7562460?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Validate that the suspicious activity was expected by the user.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "mobile":
            return False
        return bool(event.get("name") == "SUSPICIOUS_ACTIVITY_EVENT")

    def title(self, event):
        return f"User [{event.deep_get('actor', 'email', default='<UNKNOWN_USER>')}]'s device was compromised"

    tests = [
        RuleTest(
            name="Normal Mobile Event",
            expected_result=False,
            log={
                "id": {"applicationName": "mobile"},
                "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
                "type": "device_updates",
                "name": "DEVICE_SYNC_EVENT",
                "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
            },
        ),
        RuleTest(
            name="Suspicious Activity",
            expected_result=True,
            log={
                "id": {"applicationName": "mobile"},
                "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
                "type": "device_updates",
                "name": "SUSPICIOUS_ACTIVITY_EVENT",
                "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
            },
        ),
    ]



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSCloudTrailIAMEntityCreatedWithoutCloudFormation(Rule):
    id = "AWS.CloudTrail.IAMEntityCreatedWithoutCloudFormation-prototype"
    display_name = "IAM Entity Created Without CloudFormation"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    tags = ["AWS", "Configuration Required", "Identity and Access Management", "Persistence:Create Account"]
    default_severity = Severity.MEDIUM
    default_description = "An IAM Entity (Group, Policy, Role, or User) was created manually. IAM entities should be created in code to ensure that permissions are tracked and managed correctly.\n"
    default_runbook = "Verify whether IAM entity needs to exist. If so, re-create it in an appropriate CloudFormation, Terraform, or other template. Delete the original manually created entity.\n"
    default_reference = "https://blog.awsfundamentals.com/aws-iam-roles-with-aws-cloudformation"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    # The role dedicated for IAM administration
    IAM_ADMIN_ROLES = {"arn:aws:iam::123456789012:role/IdentityCFNServiceRole"}
    # The role patterns dedicated for IAM Service Roles
    IAM_ADMIN_ROLE_PATTERNS = {"arn:aws:iam::[0-9]+:role/IdentityCFNServiceRole"}
    # API calls that are indicative of IAM entity creation
    IAM_ENTITY_CREATION_EVENTS = {
        "BatchCreateUser",
        "CreateGroup",
        "CreateInstanceProfile",
        "CreatePolicy",
        "CreatePolicyVersion",
        "CreateRole",
        "CreateServiceLinkedRole",
        "CreateUser",
    }

    def rule(self, event):
        # Check if this event is in scope
        if not aws_cloudtrail_success(event) or event.get("eventName") not in self.IAM_ENTITY_CREATION_EVENTS:
            return False
        # All IAM changes MUST go through CloudFormation
        if event.deep_get("userIdentity", "invokedBy") != "cloudformation.amazonaws.com":
            return True
        # Only approved IAM Roles can make IAM Changes
        for admin_role_pattern in self.IAM_ADMIN_ROLE_PATTERNS:
            # Check if the arn matches any role patterns, return False if there is a match
            if (
                len(
                    re.findall(
                        admin_role_pattern,
                        event.deep_get("userIdentity", "sessionContext", "sessionIssuer", "arn"),
                    ),
                )
                > 0
            ):
                return False
        return event.deep_get("userIdentity", "sessionContext", "sessionIssuer", "arn") not in self.IAM_ADMIN_ROLES

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="IAM Entity Created Automatically",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/IdentityCFNServiceRole",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateUser",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"userName": "user", "path": "/"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAM Entity Created Manually With Approved Role",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/IdentityCFNServiceRole",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateUser",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"userName": "user", "path": "/"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAM Entity Created Manually With Approved Role Pattern",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::210987654321:role/IdentityCFNServiceRole",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateUser",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"userName": "user", "path": "/"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAM Entity Created Manually",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/OtherRole",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateUser",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"userName": "user", "path": "/"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Non IAM Entity Creation Event",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/OtherRole",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "NotCreateUser",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"userName": "user", "path": "/"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Error Manually Creating IAM Entity",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "errorCode": "EntityAlreadyExists",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/OtherRole",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateUser",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"userName": "user", "path": "/"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from typing import Any, Dict, Optional


def is_retryable_error(err: Optional[Dict[str, Any]]) -> bool:
    if err:
        if is_retryable_error_str(err.get("message", "")) or is_retryable_error_str(err.get("body", "")):
            return True
    return False


def is_retryable_error_str(err: str) -> bool:
    if not err:
        return False
    return "another upload" in err



from pypanther.rules.aws_cloudtrail.aws_add_malicious_lambda_extension import (
    AWSLambdaUpdateFunctionConfiguration as AWSLambdaUpdateFunctionConfiguration,
)
from pypanther.rules.aws_cloudtrail.aws_ami_modified_for_public_access import (
    AWSCloudTrailAMIModifiedForPublicAccess as AWSCloudTrailAMIModifiedForPublicAccess,
)
from pypanther.rules.aws_cloudtrail.aws_backdoor_lambda_function import (
    AWSPotentialBackdoorLambda as AWSPotentialBackdoorLambda,
)
from pypanther.rules.aws_cloudtrail.aws_bedrock_deletemodelinvocationloggingconfiguration import (
    AWSBedrockDeleteModelInvocationLoggingConfiguration as AWSBedrockDeleteModelInvocationLoggingConfiguration,
)
from pypanther.rules.aws_cloudtrail.aws_bedrock_guardrail_update_delete import (
    AWSBedrockGuardrailUpdateDelete as AWSBedrockGuardrailUpdateDelete,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_account_discovery import (
    AWSCloudTrailAccountDiscovery as AWSCloudTrailAccountDiscovery,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_attempt_to_leave_org import (
    AWSCloudTrailAttemptToLeaveOrg as AWSCloudTrailAttemptToLeaveOrg,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_created import AWSCloudTrailCreated as AWSCloudTrailCreated
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_event_selectors_disabled import (
    AWSCloudTrailEventSelectorsDisabled as AWSCloudTrailEventSelectorsDisabled,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_loginprofilecreatedormodified import (
    AWSCloudTrailLoginProfileCreatedOrModified as AWSCloudTrailLoginProfileCreatedOrModified,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_password_policy_discovery import (
    AWSCloudTrailPasswordPolicyDiscovery as AWSCloudTrailPasswordPolicyDiscovery,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_ses_check_identity_verifications import (
    AWSCloudTrailSESCheckIdentityVerifications as AWSCloudTrailSESCheckIdentityVerifications,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_ses_check_send_quota import (
    AWSCloudTrailSESCheckSendQuota as AWSCloudTrailSESCheckSendQuota,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_ses_check_ses_sending_enabled import (
    AWSCloudTrailSESCheckSESSendingEnabled as AWSCloudTrailSESCheckSESSendingEnabled,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_ses_list_identities import (
    AWSCloudTrailSESListIdentities as AWSCloudTrailSESListIdentities,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_short_lifecycle import (
    AWSCloudTrailShortLifecycle as AWSCloudTrailShortLifecycle,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_stopped import AWSCloudTrailStopped as AWSCloudTrailStopped
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_unsuccessful_mfa_attempt import (
    AWSUnsuccessfulMFAattempt as AWSUnsuccessfulMFAattempt,
)
from pypanther.rules.aws_cloudtrail.aws_cloudtrail_useraccesskeyauth import (
    AWSCloudTrailUserAccessKeyAuth as AWSCloudTrailUserAccessKeyAuth,
)
from pypanther.rules.aws_cloudtrail.aws_codebuild_made_public import (
    AWSCloudTrailCodebuildProjectMadePublic as AWSCloudTrailCodebuildProjectMadePublic,
)
from pypanther.rules.aws_cloudtrail.aws_config_service_created import AWSConfigServiceCreated as AWSConfigServiceCreated
from pypanther.rules.aws_cloudtrail.aws_config_service_disabled_deleted import (
    AWSConfigServiceDisabledDeleted as AWSConfigServiceDisabledDeleted,
)
from pypanther.rules.aws_cloudtrail.aws_console_login import AWSConsoleLogin as AWSConsoleLogin
from pypanther.rules.aws_cloudtrail.aws_console_login_without_mfa import (
    AWSConsoleLoginWithoutMFA as AWSConsoleLoginWithoutMFA,
)
from pypanther.rules.aws_cloudtrail.aws_console_login_without_saml import (
    AWSConsoleLoginWithoutSAML as AWSConsoleLoginWithoutSAML,
)
from pypanther.rules.aws_cloudtrail.aws_console_root_login import AWSConsoleRootLogin as AWSConsoleRootLogin
from pypanther.rules.aws_cloudtrail.aws_console_root_login_failed import (
    AWSConsoleRootLoginFailed as AWSConsoleRootLoginFailed,
)
from pypanther.rules.aws_cloudtrail.aws_console_signin import AWSConsoleSignIn as AWSConsoleSignIn
from pypanther.rules.aws_cloudtrail.aws_dns_logs_deleted import (
    AWSCloudTrailDNSLogsDeleted as AWSCloudTrailDNSLogsDeleted,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_download_instance_user_data import (
    AWSEC2DownloadInstanceUserData as AWSEC2DownloadInstanceUserData,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_ebs_encryption_disabled import (
    AWSEC2EBSEncryptionDisabled as AWSEC2EBSEncryptionDisabled,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_gateway_modified import AWSEC2GatewayModified as AWSEC2GatewayModified
from pypanther.rules.aws_cloudtrail.aws_ec2_launch_unusual_ec2_instances import (
    AWSEC2LaunchUnusualEC2Instances as AWSEC2LaunchUnusualEC2Instances,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_manual_security_group_changes import (
    AWSEC2ManualSecurityGroupChange as AWSEC2ManualSecurityGroupChange,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_many_passwors_read_attempts import (
    AWSEC2ManyPasswordReadAttempts as AWSEC2ManyPasswordReadAttempts,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_monitoring import AWSEC2Monitoring as AWSEC2Monitoring
from pypanther.rules.aws_cloudtrail.aws_ec2_multi_instance_connect import (
    AWSEC2MultiInstanceConnect as AWSEC2MultiInstanceConnect,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_network_acl_modified import (
    AWSEC2NetworkACLModified as AWSEC2NetworkACLModified,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_route_table_modified import (
    AWSEC2RouteTableModified as AWSEC2RouteTableModified,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_security_group_modified import (
    AWSEC2SecurityGroupModified as AWSEC2SecurityGroupModified,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_startup_script_change import (
    AWSEC2StartupScriptChange as AWSEC2StartupScriptChange,
)
from pypanther.rules.aws_cloudtrail.aws_ec2_stopinstances import AWSEC2StopInstances as AWSEC2StopInstances
from pypanther.rules.aws_cloudtrail.aws_ec2_traffic_mirroring import AWSEC2TrafficMirroring as AWSEC2TrafficMirroring
from pypanther.rules.aws_cloudtrail.aws_ec2_vpc_modified import AWSEC2VPCModified as AWSEC2VPCModified
from pypanther.rules.aws_cloudtrail.aws_ec2_vulnerable_xz_image_launched import (
    AWSEC2VulnerableXZImageLaunched as AWSEC2VulnerableXZImageLaunched,
)
from pypanther.rules.aws_cloudtrail.aws_ecr_crud import AWSECRCRUD as AWSECRCRUD
from pypanther.rules.aws_cloudtrail.aws_ecr_events import AWSECREVENTS as AWSECREVENTS
from pypanther.rules.aws_cloudtrail.aws_iam_anything_changed import (
    AWSCloudTrailIAMAnythingChanged as AWSCloudTrailIAMAnythingChanged,
)
from pypanther.rules.aws_cloudtrail.aws_iam_assume_role_blocklist_ignored import (
    AWSCloudTrailIAMAssumeRoleBlacklistIgnored as AWSCloudTrailIAMAssumeRoleBlacklistIgnored,
)
from pypanther.rules.aws_cloudtrail.aws_iam_attach_admin_role_policy import (
    AWSIAMAttachAdminRolePolicy as AWSIAMAttachAdminRolePolicy,
)
from pypanther.rules.aws_cloudtrail.aws_iam_attach_admin_user_policy import (
    AWSIAMAttachAdminUserPolicy as AWSIAMAttachAdminUserPolicy,
)
from pypanther.rules.aws_cloudtrail.aws_iam_backdoor_role import AWSIAMBackdoorRole as AWSIAMBackdoorRole
from pypanther.rules.aws_cloudtrail.aws_iam_compromised_key_quarantine import (
    AWSCloudTrailIAMCompromisedKeyQuarantine as AWSCloudTrailIAMCompromisedKeyQuarantine,
)
from pypanther.rules.aws_cloudtrail.aws_iam_create_role import AWSIAMCreateRole as AWSIAMCreateRole
from pypanther.rules.aws_cloudtrail.aws_iam_create_user import AWSIAMCreateUser as AWSIAMCreateUser
from pypanther.rules.aws_cloudtrail.aws_iam_entity_created_without_cloudformation import (
    AWSCloudTrailIAMEntityCreatedWithoutCloudFormation as AWSCloudTrailIAMEntityCreatedWithoutCloudFormation,
)
from pypanther.rules.aws_cloudtrail.aws_iam_group_read_only_events import (
    AWSIAMGroupReadOnlyEvents as AWSIAMGroupReadOnlyEvents,
)
from pypanther.rules.aws_cloudtrail.aws_iam_policy_modified import AWSIAMPolicyModified as AWSIAMPolicyModified
from pypanther.rules.aws_cloudtrail.aws_iam_user_key_created import AWSIAMBackdoorUserKeys as AWSIAMBackdoorUserKeys
from pypanther.rules.aws_cloudtrail.aws_iam_user_recon_denied import (
    AWSIAMUserReconAccessDenied as AWSIAMUserReconAccessDenied,
)
from pypanther.rules.aws_cloudtrail.aws_ipset_modified import AWSIPSetModified as AWSIPSetModified
from pypanther.rules.aws_cloudtrail.aws_key_compromised import AWSIAMAccessKeyCompromised as AWSIAMAccessKeyCompromised
from pypanther.rules.aws_cloudtrail.aws_kms_cmk_loss import AWSKMSCustomerManagedKeyLoss as AWSKMSCustomerManagedKeyLoss
from pypanther.rules.aws_cloudtrail.aws_lambda_crud import AWSLAMBDACRUD as AWSLAMBDACRUD
from pypanther.rules.aws_cloudtrail.aws_macie_evasion import AWSMacieEvasion as AWSMacieEvasion
from pypanther.rules.aws_cloudtrail.aws_modify_cloud_compute_infrastructure import (
    AWSModifyCloudComputeInfrastructure as AWSModifyCloudComputeInfrastructure,
)
from pypanther.rules.aws_cloudtrail.aws_network_acl_permissive_entry import (
    AWSCloudTrailNetworkACLPermissiveEntry as AWSCloudTrailNetworkACLPermissiveEntry,
)
from pypanther.rules.aws_cloudtrail.aws_overwrite_lambda_code import (
    AWSLambdaUpdateFunctionCode as AWSLambdaUpdateFunctionCode,
)
from pypanther.rules.aws_cloudtrail.aws_rds_manual_snapshot_created import (
    AWSRDSManualSnapshotCreated as AWSRDSManualSnapshotCreated,
)
from pypanther.rules.aws_cloudtrail.aws_rds_master_pass_updated import (
    AWSRDSMasterPasswordUpdated as AWSRDSMasterPasswordUpdated,
)
from pypanther.rules.aws_cloudtrail.aws_rds_publicrestore import AWSRDSPublicRestore as AWSRDSPublicRestore
from pypanther.rules.aws_cloudtrail.aws_rds_snapshot_shared import AWSRDSSnapshotShared as AWSRDSSnapshotShared
from pypanther.rules.aws_cloudtrail.aws_resource_made_public import (
    AWSCloudTrailResourceMadePublic as AWSCloudTrailResourceMadePublic,
)
from pypanther.rules.aws_cloudtrail.aws_root_access_key_created import (
    AWSCloudTrailRootAccessKeyCreated as AWSCloudTrailRootAccessKeyCreated,
)
from pypanther.rules.aws_cloudtrail.aws_root_activity import AWSRootActivity as AWSRootActivity
from pypanther.rules.aws_cloudtrail.aws_root_password_changed import (
    AWSCloudTrailRootPasswordChanged as AWSCloudTrailRootPasswordChanged,
)
from pypanther.rules.aws_cloudtrail.aws_s3_bucket_deleted import AWSS3BucketDeleted as AWSS3BucketDeleted
from pypanther.rules.aws_cloudtrail.aws_s3_bucket_policy_modified import (
    AWSS3BucketPolicyModified as AWSS3BucketPolicyModified,
)
from pypanther.rules.aws_cloudtrail.aws_saml_activity import AWSSuspiciousSAMLActivity as AWSSuspiciousSAMLActivity
from pypanther.rules.aws_cloudtrail.aws_secretsmanager_retrieve_secrets import (
    AWSSecretsManagerRetrieveSecrets as AWSSecretsManagerRetrieveSecrets,
)
from pypanther.rules.aws_cloudtrail.aws_secretsmanager_retrieve_secrets_batch import (
    AWSSecretsManagerBatchRetrieveSecrets as AWSSecretsManagerBatchRetrieveSecrets,
)
from pypanther.rules.aws_cloudtrail.aws_secretsmanager_retrieve_secrets_catchall import (
    AWSSecretsManagerBatchRetrieveSecretsCatchAll as AWSSecretsManagerBatchRetrieveSecretsCatchAll,
)
from pypanther.rules.aws_cloudtrail.aws_secretsmanager_retrieve_secrets_multiregion import (
    AWSSecretsManagerRetrieveSecretsMultiRegion as AWSSecretsManagerRetrieveSecretsMultiRegion,
)
from pypanther.rules.aws_cloudtrail.aws_security_configuration_change import (
    AWSCloudTrailSecurityConfigurationChange as AWSCloudTrailSecurityConfigurationChange,
)
from pypanther.rules.aws_cloudtrail.aws_securityhub_finding_evasion import (
    AWSSecurityHubFindingEvasion as AWSSecurityHubFindingEvasion,
)
from pypanther.rules.aws_cloudtrail.aws_snapshot_made_public import (
    AWSCloudTrailSnapshotMadePublic as AWSCloudTrailSnapshotMadePublic,
)
from pypanther.rules.aws_cloudtrail.aws_software_discovery import AWSSoftwareDiscovery as AWSSoftwareDiscovery
from pypanther.rules.aws_cloudtrail.aws_unauthorized_api_call import (
    AWSCloudTrailUnauthorizedAPICall as AWSCloudTrailUnauthorizedAPICall,
)
from pypanther.rules.aws_cloudtrail.aws_unused_region import AWSUnusedRegion as AWSUnusedRegion
from pypanther.rules.aws_cloudtrail.aws_update_credentials import AWSIAMCredentialsUpdated as AWSIAMCredentialsUpdated
from pypanther.rules.aws_cloudtrail.aws_vpc_flow_logs_deleted import AWSVPCFlowLogsDeleted as AWSVPCFlowLogsDeleted
from pypanther.rules.aws_cloudtrail.aws_waf_disassociation import AWSWAFDisassociation as AWSWAFDisassociation
from pypanther.rules.aws_cloudtrail.retrieve_sso_access_token import RetrieveSSOaccesstoken as RetrieveSSOaccesstoken
from pypanther.rules.aws_cloudtrail.role_assumed_by_aws_service import (
    RoleAssumedbyAWSService as RoleAssumedbyAWSService,
)
from pypanther.rules.aws_cloudtrail.role_assumed_by_user import RoleAssumedbyUser as RoleAssumedbyUser
from pypanther.rules.aws_cloudtrail.signin_with_aws_cli_prompt import SigninwithAWSCLIprompt as SigninwithAWSCLIprompt



from pypanther.rules.okta.okta_account_support_access import OktaSupportAccess as OktaSupportAccess
from pypanther.rules.okta.okta_admin_disabled_mfa import OktaGlobalMFADisabled as OktaGlobalMFADisabled
from pypanther.rules.okta.okta_admin_role_assigned import OktaAdminRoleAssigned as OktaAdminRoleAssigned
from pypanther.rules.okta.okta_anonymizing_vpn_login import OktaAnonymizingVPNLogin as OktaAnonymizingVPNLogin
from pypanther.rules.okta.okta_api_key_created import OktaAPIKeyCreated as OktaAPIKeyCreated
from pypanther.rules.okta.okta_api_key_revoked import OktaAPIKeyRevoked as OktaAPIKeyRevoked
from pypanther.rules.okta.okta_app_refresh_access_token_reuse import (
    OktaRefreshAccessTokenReuse as OktaRefreshAccessTokenReuse,
)
from pypanther.rules.okta.okta_app_unauthorized_access_attempt import (
    OktaAppUnauthorizedAccessAttempt as OktaAppUnauthorizedAccessAttempt,
)
from pypanther.rules.okta.okta_group_admin_role_assigned import OktaGroupAdminRoleAssigned as OktaGroupAdminRoleAssigned
from pypanther.rules.okta.okta_idp_create_modify import (
    OktaIdentityProviderCreatedModified as OktaIdentityProviderCreatedModified,
)
from pypanther.rules.okta.okta_idp_signin import OktaIdentityProviderSignIn as OktaIdentityProviderSignIn
from pypanther.rules.okta.okta_login_signal import OktaLoginSuccess as OktaLoginSuccess
from pypanther.rules.okta.okta_login_without_push_marker import OktaLoginWithoutPushMarker as OktaLoginWithoutPushMarker
from pypanther.rules.okta.okta_new_behavior_accessing_admin_console import (
    OktaNewBehaviorAccessingAdminConsole as OktaNewBehaviorAccessingAdminConsole,
)
from pypanther.rules.okta.okta_org2org_creation_modification import (
    OktaOrg2orgCreationModification as OktaOrg2orgCreationModification,
)
from pypanther.rules.okta.okta_password_accessed import OktaPasswordAccess as OktaPasswordAccess
from pypanther.rules.okta.okta_password_extraction_via_scim import (
    OktaPasswordExtractionviaSCIM as OktaPasswordExtractionviaSCIM,
)
from pypanther.rules.okta.okta_phishing_attempt_blocked_by_fastpass import (
    OktaPhishingAttemptBlockedFastPass as OktaPhishingAttemptBlockedFastPass,
)
from pypanther.rules.okta.okta_potentially_stolen_session import (
    OktaPotentiallyStolenSession as OktaPotentiallyStolenSession,
)
from pypanther.rules.okta.okta_rate_limits import OktaRateLimits as OktaRateLimits
from pypanther.rules.okta.okta_sso_to_aws import OktaSSOtoAWS as OktaSSOtoAWS
from pypanther.rules.okta.okta_support_reset import OktaSupportReset as OktaSupportReset
from pypanther.rules.okta.okta_threatinsight_security_threat_detected import (
    OktaThreatInsightSecurityThreatDetected as OktaThreatInsightSecurityThreatDetected,
)
from pypanther.rules.okta.okta_user_account_locked import OktaUserAccountLocked as OktaUserAccountLocked
from pypanther.rules.okta.okta_user_mfa_factor_suspend import OktaUserMFAFactorSuspend as OktaUserMFAFactorSuspend
from pypanther.rules.okta.okta_user_mfa_reset import OktaUserMFAResetSingle as OktaUserMFAResetSingle
from pypanther.rules.okta.okta_user_mfa_reset_all import OktaUserMFAResetAll as OktaUserMFAResetAll
from pypanther.rules.okta.okta_user_reported_suspicious_activity import (
    OktaUserReportedSuspiciousActivity as OktaUserReportedSuspiciousActivity,
)



import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_walk
from pypanther.helpers.config import config
from pypanther.helpers.gcp import get_k8s_info


@panther_managed
class GCPK8sExecIntoPod(Rule):
    id = "GCP.K8s.ExecIntoPod-prototype"
    display_name = "Exec into Pod"
    enabled = False
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Security Control", "Configuration Required"]
    default_severity = Severity.MEDIUM
    default_description = "Alerts when users exec into pod. Possible to specify specific projects and allowed users.\n"
    default_runbook = "Investigate the user and determine why. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://cloud.google.com/migrate/containers/docs/troubleshooting/executing-shell-commands"
    GCP_PRODUCTION_PROJECT_IDS = config.GCP_PRODUCTION_PROJECT_IDS
    # This is a list of principals that are allowed to exec into pods
    # in various namespaces and projects.
    # If empty, then no principals
    # "system:serviceaccount:example-namespace:example-namespace-service-account",
    # If empty, then all namespaces
    # If projects empty then all projects
    # Add more allowed principals here
    # {
    #     "principals": [],
    #     "namespaces": [],
    #     "projects": [],
    # },
    ALLOW_LIST = [{"principals": [], "namespaces": [], "projects": []}]

    def rule(self, event):
        # pylint: disable=not-callable
        if isinstance(self.ALLOW_LIST, MagicMock):
            self.ALLOW_LIST = json.loads(self.ALLOW_LIST())
        # Defaults to False (no alert) unless method is exec and principal not allowed
        if not all(
            [
                event.deep_walk("protoPayload", "methodName") == "io.k8s.core.v1.pods.exec.create",
                event.deep_walk("resource", "type") == "k8s_cluster",
            ],
        ):
            return False
        k8s_info = get_k8s_info(event)
        principal = deep_walk(k8s_info, "principal", default="<NO PRINCIPAL>")
        namespace = deep_walk(k8s_info, "namespace", default="<NO NAMESPACE>")
        project_id = deep_walk(k8s_info, "project_id", default="<NO PROJECT_ID>")
        # rule_exceptions that are allowed temporarily are defined in gcp_environment.py
        # Some execs have principal which is long numerical UUID, appears to be k8s internals
        for allowed_principal in self.ALLOW_LIST:
            allowed_principals = deep_walk(allowed_principal, "principals", default=[])
            allowed_namespaces = deep_walk(allowed_principal, "namespaces", default=[])
            allowed_project_ids = deep_walk(allowed_principal, "projects", default=[])
            if (
                principal in allowed_principals
                and (namespace in allowed_namespaces or allowed_namespaces == [])
                and (project_id in allowed_project_ids or allowed_project_ids == [])
            ):
                if "@" not in principal:
                    return False
        return True

    def severity(self, event):
        project_id = deep_walk(get_k8s_info(event), "project_id", default="<NO PROJECT_ID>")
        if project_id in self.GCP_PRODUCTION_PROJECT_IDS:
            return "high"
        return "info"

    def title(self, event):
        # TODO: use unified data model field in title for actor
        k8s_info = get_k8s_info(event)
        principal = deep_walk(k8s_info, "principal", default="<NO PRINCIPAL>")
        project_id = deep_walk(k8s_info, "project_id", default="")
        pod = deep_walk(k8s_info, "pod", default="")
        namespace = deep_walk(k8s_info, "namespace", default="")
        return f"Exec into pod namespace/{namespace}/pod/{pod} by {principal} in {project_id}"

    def alert_context(self, event):
        return get_k8s_info(event)

    tests = [
        RuleTest(
            name="Allowed User",
            expected_result=False,
            mocks=[
                RuleMock(
                    object_name="ALLOW_LIST",
                    return_value='[\n  {\n    "principals": ["system:serviceaccount:example-namespace:example-namespace-service-account"],\n    "namespaces": [],\n    "projects": []\n  }\n]',
                ),
            ],
            log={
                "protoPayload": {
                    "authenticationInfo": {
                        "principalEmail": "system:serviceaccount:example-namespace:example-namespace-service-account",
                    },
                    "authorizationInfo": [
                        {
                            "permission": "io.k8s.core.v1.pods.exec.create",
                            "resource": "core/v1/namespaces/opa/pods/opa-57998cf7c5-bjkfk/exec",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.exec.create",
                    "requestMetadata": {
                        "callerIp": "88.88.88.88",
                        "callerSuppliedUserAgent": "kubectl/v1.40.8 (darwin/amd64) kubernetes/6575935",
                    },
                    "resourceName": "core/v1/namespaces/example/pods/one-off-46666967280/exec",
                    "timestamp": "2022-03-04T16:01:49.978756Z",
                },
                "resource": {"type": "k8s_cluster", "labels": {"project_id": "rigup-production"}},
            },
        ),
        RuleTest(
            name="Disallowed User",
            expected_result=True,
            log={
                "protoPayload": {
                    "authenticationInfo": {"principalEmail": "disallowed.user@example.com"},
                    "authorizationInfo": [
                        {
                            "permission": "io.k8s.core.v1.pods.exec.create",
                            "resource": "core/v1/namespaces/example/pods/example-57998cf7c5-bjkfk/exec",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.exec.create",
                    "requestMetadata": {
                        "callerIp": "88.88.88.88",
                        "callerSuppliedUserAgent": "kubectl/v1.40.8 (darwin/amd64) kubernetes/6575935",
                    },
                    "resourceName": "core/v1/namespaces/example/pods/one-off-valerii-tovstyk-1646666967280/exec",
                    "timestamp": "2022-03-04T16:01:49.978756Z",
                },
                "resource": {"type": "k8s_cluster", "labels": {"project_id": "rigup-production"}},
            },
        ),
        RuleTest(
            name="Disallowed User2 - not an allowed namespace",
            expected_result=True,
            log={
                "protoPayload": {
                    "authenticationInfo": {"principalEmail": "example-allowed-user@example.com"},
                    "authorizationInfo": [
                        {
                            "permission": "io.k8s.core.v1.pods.exec.create",
                            "resource": "core/v1/namespaces/istio-system/pods/opa-57998cf7c5-bjkfk/exec",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.exec.create",
                    "requestMetadata": {
                        "callerIp": "88.88.88.88",
                        "callerSuppliedUserAgent": "kubectl/v1.40.8 (darwin/amd64) kubernetes/6575935",
                    },
                    "resourceName": "core/v1/namespaces/istio-system/pods/one-off-valerii-tovstyk-1646666967280/exec",
                    "timestamp": "2022-03-04T16:01:49.978756Z",
                },
                "resource": {"type": "k8s_cluster", "labels": {"project_id": "rigup-production"}},
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class DUOUserBypassCodeUsed(Rule):
    id = "DUO.User.BypassCode.Used-prototype"
    display_name = "Duo User Bypass Code Used"
    dedup_period_minutes = 5
    log_types = [LogType.DUO_AUTHENTICATION]
    tags = ["Duo"]
    default_severity = Severity.LOW
    default_description = "A Duo user's bypass code was used to authenticate"
    default_reference = "https://duo.com/docs/adminapi#authentication-logs"
    default_runbook = "Follow up with the user to confirm they used the bypass code themselves."

    def rule(self, event):
        return event.get("reason") == "bypass_user" and event.get("result") == "success"

    def title(self, event):
        user = event.deep_get("user", "name", default="Unknown")
        return f"Bypass code for Duo User [{user}] used"

    def alert_context(self, event):
        return {
            "factor": event.get("factor"),
            "reason": event.get("reason"),
            "user": event.deep_get("user", "name", default=""),
            "os": event.deep_get("access_device", "os", default=""),
            "ip_access": event.deep_get("access_device", "ip", default=""),
            "ip_auth": event.deep_get("auth_device", "ip", default=""),
            "application": event.deep_get("application", "name", default=""),
        }

    tests = [
        RuleTest(
            name="bypass_code_used",
            expected_result=True,
            log={
                "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
                "auth_device": {"ip": "12.12.12.12"},
                "application": {"key": "D12345", "name": "Slack"},
                "event_type": "authentication",
                "factor": "duo_push",
                "reason": "bypass_user",
                "result": "success",
                "user": {"name": "example@example.io"},
            },
        ),
        RuleTest(
            name="good_auth",
            expected_result=False,
            log={
                "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
                "auth_device": {"ip": "12.12.12.12"},
                "application": {"key": "D12345", "name": "Slack"},
                "event_type": "authentication",
                "factor": "duo_push",
                "reason": "user_approved",
                "result": "success",
                "user": {"name": "example@example.io"},
            },
        ),
        RuleTest(
            name="denied_old_creds",
            expected_result=False,
            log={
                "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
                "auth_device": {"ip": "12.12.12.12"},
                "application": {"key": "D12345", "name": "Slack"},
                "event_type": "authentication",
                "factor": "duo_push",
                "reason": "out_of_date",
                "result": "denied",
                "user": {"name": "example@example.io"},
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPVPCFlowLogsDisabled(Rule):
    default_description = "VPC flow logs were disabled for a subnet."
    display_name = "GCP VPC Flow Logs Disabled"
    default_reference = "https://cloud.google.com/vpc/docs/using-flow-logs"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.VPC.Flow.Logs.Disabled-prototype"

    def rule(self, event):
        return all(
            [
                event.get("protoPayload"),
                event.deep_get("protoPayload", "methodName", default="") == "v1.compute.subnetworks.patch",
                event.deep_get("protoPayload", "request", "enableFlowLogs") is False,
            ],
        )

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"GCP: [{actor}] disabled VPC Flow Logs for [{resource}]"

    tests = [
        RuleTest(
            name="Disable Flow Logs Event",
            expected_result=True,
            log={
                "insertId": "123456",
                "logName": "projects/gcp-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "operation-abc-123",
                    "last": True,
                    "producer": "compute.googleapis.com",
                },
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-08 18:52:58.322",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-08 18:54:14.597",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user1@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "compute.subnetworks.update",
                            "resourceAttributes": {
                                "name": "projects/gcp-project/regions/us-central1/subnetworks/default",
                                "service": "compute",
                                "type": "compute.subnetworks",
                            },
                        },
                    ],
                    "methodName": "v1.compute.subnetworks.patch",
                    "request": {
                        "@type": "type.googleapis.com/compute.subnetworks.patch",
                        "enableFlowLogs": False,
                        "fingerprint": "/�/��\x03��",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {
                            "auth": {},
                            "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                            "time": "2023-03-08T18:52:58.731721Z",
                        },
                    },
                    "resourceName": "projects/gcp-project/regions/us-central1/subnetworks/default",
                    "response": {
                        "@type": "type.googleapis.com/operation",
                        "endTime": "2023-03-08T10:52:58.700-08:00",
                        "id": "123456",
                        "insertTime": "2023-03-08T10:52:58.699-08:00",
                        "name": "operation-1678301578299-5f668096688bc-0635b6ef-4d0122bb",
                        "operationType": "compute.subnetworks.patch",
                        "progress": "100",
                        "region": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1",
                        "selfLink": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1/operations/1234",
                        "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1/operations/1234",
                        "startTime": "2023-03-08T10:52:58.700-08:00",
                        "status": "DONE",
                        "targetId": "123456",
                        "targetLink": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1/subnetworks/default",
                        "user": "user1@company.io",
                    },
                    "serviceName": "compute.googleapis.com",
                },
                "receiveTimestamp": "2023-03-08 18:52:58.991",
                "resource": {
                    "labels": {
                        "location": "us-central1",
                        "project_id": "gcp-project",
                        "subnetwork_id": "123456",
                        "subnetwork_name": "default",
                    },
                    "type": "gce_subnetwork",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-08 18:52:58.322",
            },
        ),
        RuleTest(
            name="Enable Flow Logs Event",
            expected_result=False,
            log={
                "insertId": "12345",
                "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "operation-abcdefg-1234567",
                    "last": True,
                    "producer": "compute.googleapis.com",
                },
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-08 18:52:52.114",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-08 18:54:14.595",
                "p_row_id": "5e7586fcbb73fdeed985ebeb16bd0c",
                "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user1@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "compute.subnetworks.update",
                            "resourceAttributes": {
                                "name": "projects/test-project/regions/us-central1/subnetworks/default",
                                "service": "compute",
                                "type": "compute.subnetworks",
                            },
                        },
                    ],
                    "methodName": "v1.compute.subnetworks.patch",
                    "request": {
                        "@type": "type.googleapis.com/compute.subnetworks.patch",
                        "fingerprint": "/�/��\x03��",
                        "logConfig": {"enable": True},
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {
                            "auth": {},
                            "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                            "time": "2023-03-08T18:52:52.558899Z",
                        },
                    },
                    "resourceName": "projects/test-project/regions/us-central1/subnetworks/default",
                    "response": {
                        "@type": "type.googleapis.com/operation",
                        "endTime": "2023-03-08T10:52:52.510-08:00",
                        "id": "9876543210",
                        "insertTime": "2023-03-08T10:52:52.509-08:00",
                        "name": "operation-abcdefg-1234567",
                        "operationType": "compute.subnetworks.patch",
                        "progress": "100",
                        "region": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1",
                        "selfLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/operation-abcdefg-1234567",
                        "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/9876543210",
                        "startTime": "2023-03-08T10:52:52.510-08:00",
                        "status": "DONE",
                        "targetId": "567854321",
                        "targetLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/subnetworks/default",
                        "user": "user1@company.io",
                    },
                    "serviceName": "compute.googleapis.com",
                },
                "receiveTimestamp": "2023-03-08 18:52:52.72",
                "resource": {
                    "labels": {
                        "location": "us-central1",
                        "project_id": "test-project",
                        "subnetwork_id": "567854321",
                        "subnetwork_name": "default",
                    },
                    "type": "gce_subnetwork",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-08 18:52:52.114",
            },
        ),
    ]
